#####################################################################################
# The MIT License (MIT)
#
# Copyright (c) 2015-2022 Advanced Micro Devices, Inc. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#####################################################################################
import migraphx, struct, array, sys
try:
    from functools import reduce
except:
    pass


def assert_eq(x, y):
    if x == y:
        pass
    else:
        raise Exception(str(x) + " != " + str(y))


def read_float(b, index):
    return struct.unpack_from('f', b, index * 4)[0]


def write_float(b, index):
    struct.pack_into('f', b, index * 4)


def nelements(lens):
    return reduce(lambda x, y: x * y, lens, 1)


def create_buffer(t, data, shape):
    a = array.array(t, data)
    if sys.version_info >= (3, 0):
        m = memoryview(a.tobytes())
        return m.cast(t, shape)
    else:
        m = memoryview(a.tostring())
        return m


def check_argument(a):
    l = a.tolist()
    for i in range(len(l)):
        assert_eq(l[i], read_float(a, i))


def check_shapes(r, m):
    lens = list(m.shape)
    strides = [int(s / m.itemsize) for s in m.strides]
    elements = nelements(lens)
    assert_eq(r.get_shape().elements(), elements)
    assert_eq(r.get_shape().lens(), lens)
    assert_eq(r.get_shape().strides(), strides)


def run(p):
    params = {}
    for key, value in p.get_parameter_shapes().items():
        params[key] = migraphx.generate_argument(value)

    return p.run(params)


def test_shape(shape):
    data = list(range(nelements(shape)))
    m = create_buffer('f', data, shape)
    a = migraphx.argument(m)
    check_shapes(a, m)
    assert_eq(a.tolist(), data)


def test_input():
    if sys.version_info >= (3, 0):
        test_shape([4])
        test_shape([2, 3])
    else:
        data = list(range(4))
        m = create_buffer('f', data, [4])
        a1 = migraphx.argument(m)
        a2 = migraphx.argument(bytearray(a1))
        check_shapes(a2, m)
        assert_eq(a1.tolist(), m.tolist())


def test_output():
    p = migraphx.parse_onnx("conv_relu_maxpool_test.onnx")
    p.compile(migraphx.get_target("gpu"))

    r1 = run(p)[-1]
    r2 = run(p)[-1]
    assert_eq(r1, r2)
    assert_eq(r1.tolist(), r2.tolist())

    check_argument(r1)
    check_argument(r2)

    m1 = memoryview(r1)
    m2 = memoryview(r2)

    check_shapes(r1, m1)
    check_shapes(r2, m2)


test_input()
test_output()



#####################################################################################
# The MIT License (MIT)
#
# Copyright (c) 2015-2024 Advanced Micro Devices, Inc. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#####################################################################################

from asr import WhisperMGX
import gradio as gr
import os


def main():
    # Note: This will load the models, which can take several minutes
    w = WhisperMGX()

    def gr_wrapper(audio):
        data, fr = WhisperMGX.load_audio_from_file(audio)
        input_features = w.get_input_features_from_sample(data, fr)
        return w.generate(input_features)

    examples = [
        os.path.join(os.path.dirname(__file__), "audio/sample1.flac"),
        os.path.join(os.path.dirname(__file__), "audio/sample2.flac"),
    ]
    # skip if there is no file
    examples = [e for e in examples if os.path.isfile(e)]

    demo = gr.Interface(
        gr_wrapper,
        gr.Audio(sources=["upload", "microphone"], type="filepath"),
        "text",
        examples=examples,
    )
    demo.launch()


if __name__ == "__main__":
    main()



#####################################################################################
# The MIT License (MIT)
#
# Copyright (c) 2015-2025 Advanced Micro Devices, Inc. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#####################################################################################

from argparse import ArgumentParser
from flux_pipeline import FluxPipeline


def get_args():
    parser = ArgumentParser()

    parser.add_argument(
        "--hf-model",
        type=str,
        choices=[
            "black-forest-labs/FLUX.1-dev",
            "black-forest-labs/FLUX.1-schnell",
        ],
        default="black-forest-labs/FLUX.1-dev",
        help=
        "Specify HF model card. Options: 'black-forest-labs/FLUX.1-dev', 'black-forest-labs/FLUX.1-schnell'",
    )

    parser.add_argument(
        "--local-dir",
        type=str,
        default=None,
        help="Specify directory with local onnx files (or where to export)",
    )

    parser.add_argument(
        "--compile-dir",
        type=str,
        default=None,
        help="Specify directory with compile mxr files (or where to export)",
    )

    parser.add_argument(
        "-d",
        "--image-height",
        type=int,
        default=1024,
        help="Output Image height, default 1024",
    )

    parser.add_argument(
        "-w",
        "--image-width",
        type=int,
        default=1024,
        help="Output Image width, default 1024",
    )

    parser.add_argument(
        "-g",
        "--guidance-scale",
        type=float,
        default=3.5,
        help="Guidance scale, default 3.5",
    )

    parser.add_argument(
        "-l",
        "--max-sequence-length",
        type=int,
        default=512,
        help="Max sequence length for T5, default 512",
    )

    parser.add_argument(
        "-p",
        "--prompt",
        default=["A cat holding a sign that says hello world"],
        nargs="*",
        help="Text prompt(s) to be sent to the CLIP tokenizer and text encoder",
    )

    parser.add_argument(
        "--prompt2",
        default=None,
        nargs="*",
        help=
        "Text prompt(s) to be sent to the T5 tokenizer and text encoder. If not defined, prompt will be used instead",
    )

    parser.add_argument(
        "-s",
        "--denoising-steps",
        type=int,
        default=50,
        help="Number of denoising steps",
    )

    parser.add_argument("--fp16",
                        action='store_true',
                        help="Apply fp16 quantization.")

    parser.add_argument("--bf16",
                        action='store_true',
                        help="Apply bf16 quantization.")

    parser.add_argument(
        "--output-dir",
        type=str,
        default="./",
        help="Specify directory where images should be saved",
    )

    parser.add_argument(
        "-o",
        "--output-prefix",
        type=str,
        default="flux",
        help="Specify image name prefix for saving result images",
    )

    parser.add_argument(
        "-b",
        "--benchmark-runs",
        type=int,
        default=None,
        help="Number of runs to do for benchmarking. Default: no benchmarking",
    )

    parser.add_argument("--exhaustive-tune",
                        action='store_true',
                        help="Perform exhaustive tuning when compiling")

    parser.add_argument(
        "--batch-size",
        type=int,
        default=None,
        help=
        "Set custom batch size (expects len 1 prompt, useful for benchmarking)"
    )

    return parser.parse_args()


if __name__ == "__main__":
    args = get_args()

    prompt = args.prompt
    prompt2 = args.prompt2 if args.prompt2 else prompt

    if args.batch_size:
        assert len(prompt) == 1 and len(prompt2) == 1
        prompt = prompt * args.batch_size
        prompt2 = prompt2 * args.batch_size

    bf16 = args.bf16
    fp16 = args.fp16 and not bf16

    pipe = FluxPipeline(hf_model_path=args.hf_model,
                        local_dir=args.local_dir,
                        compile_dir=args.compile_dir,
                        img_height=args.image_height,
                        img_width=args.image_width,
                        guidance_scale=args.guidance_scale,
                        max_sequence_length=args.max_sequence_length,
                        batch_size=len(prompt),
                        denoising_steps=args.denoising_steps,
                        fp16=fp16,
                        bf16=bf16,
                        exhaustive_tune=args.exhaustive_tune)

    pipe.load_models()

    images = pipe.infer(prompt, prompt2, warmup=True)

    if args.output_dir:
        print(f"Saving images to {args.output_dir}")
        pipe.save_image(images, args.output_prefix, args.output_dir)

    if args.benchmark_runs:
        pipe.clear_run_data()
        print("Begin benchmarking...")
        for _ in range(args.benchmark_runs):
            pipe.infer(prompt, prompt2)
            print(f"Run time: {pipe.times[-1]}s")

        pipe.print_summary()



#####################################################################################
# The MIT License (MIT)
#
# Copyright (c) 2015-2022 Advanced Micro Devices, Inc. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#####################################################################################
import migraphx, array, sys


def create_buffer(t, data, shape):
    a = array.array(t, data)
    m = memoryview(a.tobytes())
    return m.cast(t, shape)


def test_add_op():
    p = migraphx.program()
    mm = p.get_main_module()
    x = mm.add_literal(create_buffer('f', [1.0] * 9, (3, 3)))
    y = mm.add_literal(create_buffer('f', [2.0] * 9, (3, 3)))
    add_op = mm.add_instruction(migraphx.op("add"), [x, y])
    mm.add_return([add_op])
    p.compile(migraphx.get_target("ref"))
    params = {}
    output = p.run(params)[-1].tolist()
    assert output == list([3.0] * 9)


def test_if_then_else():
    param_shape = migraphx.shape(lens=[3, 3], type="float")
    cond_shape = migraphx.shape(type="bool", lens=[1], strides=[0])

    def create_program():
        p = migraphx.program()
        mm = p.get_main_module()
        cond = mm.add_parameter("cond", cond_shape)
        x = mm.add_parameter("x", param_shape)
        y = mm.add_parameter("y", param_shape)
        then_mod = p.create_module("If_0_if")
        x_identity = then_mod.add_instruction(migraphx.op("identity"), [x])
        then_mod.add_return([x_identity])

        else_mod = p.create_module("If_0_else")
        y_identity = else_mod.add_instruction(migraphx.op("identity"), [y])
        else_mod.add_return([y_identity])

        if_ins = mm.add_instruction(migraphx.op("if"), [cond],
                                    [then_mod, else_mod])
        ret = mm.add_instruction(migraphx.op("get_tuple_elem", **{"index": 0}),
                                 [if_ins])
        mm.add_return([ret])
        return p

    params = {}
    params["x"] = migraphx.generate_argument(param_shape)
    params["y"] = migraphx.generate_argument(param_shape)

    def run_prog(cond):
        p = create_program()
        p.compile(migraphx.get_target("ref"))
        params["cond"] = migraphx.fill_argument(cond_shape, cond)
        output = p.run(params)[-1]
        return output

    assert run_prog(True) == params["x"]
    assert run_prog(False) == params["y"]


if __name__ == "__main__":
    if sys.version_info >= (3, 0):
        test_add_op()
    test_if_then_else()



#####################################################################################
# The MIT License (MIT)
#
# Copyright (c) 2015-2022 Advanced Micro Devices, Inc. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#####################################################################################
"""
Implements ONNX's backend API.
"""
import sys
if sys.version_info < (3, 0):
    sys.exit()

import migraphx
from onnx.backend.base import BackendRep
import numpy as np
from typing import Any, Tuple


class MIGraphXBackendRep(BackendRep):
    """
    Computes the prediction for a pipeline converted into
    an :class:`onnxruntime.InferenceSession` node.
    """
    def __init__(self, prog, input_names):
        """
        :param session: :class:`migraphx.program`
        """
        self._program = prog
        self._input_names = input_names

    def run(self, inputs, **kwargs):  # type: (Any, **Any) -> Tuple[Any, ...]
        """
        Computes the prediction.
        See :meth:`migraphx.program.run`.
        """

        if isinstance(inputs, list):
            inps = {}
            for i, name in enumerate(self._input_names):
                inps[name] = migraphx.argument(inputs[i])
            mgx_outputs = self._program.run(inps)
            outs = []
            for out in mgx_outputs:
                outs.append(np.array(out))
            return outs
        else:
            inp = self._program.get_parameter_shapes().keys()
            if len(inp) != 1:
                raise RuntimeError("Model expect {0} inputs".format(len(inp)))
            inps = {inp[0]: migraphx.argument(inputs)}
            mgx_outputs = self._program.run(inps)
            outs = []
            for out in mgx_outputs:
                outs.append(np.array(out))
            return self._program.run(inps)



#####################################################################################
# The MIT License (MIT)
#
# Copyright (c) 2015-2022 Advanced Micro Devices, Inc. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#####################################################################################
import argparse
import onnx
from onnx import version_converter


def parse_args():
    parser = argparse.ArgumentParser(
        description=
        'MIGraphX Onnx Model Convertion. Use to convert the opset of the input model to MIGraphX\'s'
    )
    req_args = parser.add_argument_group(title='required arguments')
    req_args.add_argument('--model',
                          type=str,
                          required=True,
                          help='path to onnx file')
    req_args.add_argument('--output',
                          type=str,
                          required=True,
                          help='path to output onnx file')
    req_args.add_argument('--opset',
                          type=int,
                          required=True,
                          help='The output opset')
    req_args.add_argument('--infer_shapes',
                          action='store_true',
                          help='Infer shapes for output model')
    parser.add_argument('--verbose',
                        action='store_true',
                        help='show verbose information (for debugging)')
    args = parser.parse_args()

    return args


def main():
    args = parse_args()

    model_path = args.model
    out_model_path = args.output
    target_opset = args.opset
    verbose = args.verbose
    infer_shapes = args.infer_shapes

    original_model = onnx.load(model_path)
    if verbose:
        print(f"The model before conversion:\n{original_model}")

    # A full list of supported adapters can be found here:
    # https://github.com/onnx/onnx/blob/main/onnx/version_converter.py#L21
    # Apply the version conversion on the original model
    converted_model = version_converter.convert_version(
        original_model, target_opset)

    if infer_shapes:
        converted_model = onnx.shape_inference.infer_shapes(converted_model)

    if verbose:
        print(f"The model after conversion:\n{converted_model}")

    # Save the ONNX model
    onnx.save(converted_model, out_model_path)


if __name__ == '__main__':
    main()



#####################################################################################
# The MIT License (MIT)
#
# Copyright (c) 2015-2025 Advanced Micro Devices, Inc. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#####################################################################################
import os, sys
import numpy as np
import argparse
import onnx
from onnx import numpy_helper
import migraphx


def parse_args():
    parser = argparse.ArgumentParser(description="MIGraphX test runner")
    parser.add_argument('test_dir',
                        type=str,
                        metavar='test_loc',
                        help='folder where the test is stored')
    parser.add_argument('--target',
                        type=str,
                        default='gpu',
                        help='Specify where the tests execute (ref, gpu)')
    parser.add_argument('--fp16', action='store_true', help='Quantize to fp16')
    parser.add_argument('--atol',
                        type=float,
                        default=1e-3,
                        help='The absolute tolerance parameter')
    parser.add_argument('--rtol',
                        type=float,
                        default=1e-3,
                        help='The relative tolerance parameter')
    args = parser.parse_args()

    return args


def get_sub_folders(dir_name):
    dir_contents = os.listdir(dir_name)
    folders = []
    for item in dir_contents:
        tmp_item = dir_name + '/' + item
        if os.path.isdir(tmp_item):
            folders.append(item)
    folders.sort()

    return folders


def get_test_cases(dir_name):
    return get_sub_folders(dir_name)


def get_model_name(dir_name):
    dir_contents = os.listdir(dir_name)
    for item in dir_contents:
        file_name = dir_name + '/' + item
        if os.path.isfile(file_name) and file_name.endswith('.onnx'):
            return item

    return ''


def read_pb_file(filename):
    with open(filename, 'rb') as pfile:
        data_str = pfile.read()
        tensor = onnx.TensorProto()
        tensor.ParseFromString(data_str)
        np_array = numpy_helper.to_array(tensor)

    return tensor.name, np_array


def wrapup_inputs(io_folder, param_names):
    param_map = {}
    data_array = []
    name_array = []
    for i in range(len(param_names)):
        file_name = io_folder + '/input_' + str(i) + '.pb'
        name, data = read_pb_file(file_name)
        param_map[name] = data
        data_array.append(data)
        if name:
            name_array.append(name)

    if len(name_array) < len(data_array):
        param_map = {}
        for i in range(len(param_names)):
            param_map[param_names[i]] = data_array[i]

        return param_map

    for name in param_names:
        if not name in param_map.keys():
            print("Input {} does not exist!".format(name))
            sys.exit()

    return param_map


def read_outputs(io_folder, out_names):
    outputs = []
    data_array = []
    name_array = []
    for i in range(len(out_names)):
        file_name = io_folder + '/output_' + str(i) + '.pb'
        name, data = read_pb_file(file_name)
        data_array.append(data)
        if name:
            name_array.append(name)

    if len(name_array) < len(data_array):
        return data_array

    for name in out_names:
        index = name_array.index(name)
        outputs.append(data_array[index])

    return outputs


def model_parameter_names(model_file_name):
    with open(model_file_name, 'rb') as pfile:
        data_str = pfile.read()
        model_proto = onnx.ModelProto()
        model_proto.ParseFromString(data_str)
        init_names = set([(i.name) for i in model_proto.graph.initializer])
        param_names = [
            input.name for input in model_proto.graph.input
            if input.name not in init_names
        ]

        return param_names


def model_output_names(model_file_name):
    with open(model_file_name, 'rb') as pfile:
        data_str = pfile.read()
        model_proto = onnx.ModelProto()
        model_proto.ParseFromString(data_str)
        output_names = [out.name for out in model_proto.graph.output]

        return output_names


def get_input_shapes(sample_case, param_names):
    param_shape_map = {}
    name_array = []
    shape_array = []
    for i in range(len(param_names)):
        file_name = sample_case + '/input_' + str(i) + '.pb'
        name, data = read_pb_file(file_name)
        param_shape_map[name] = data.shape
        shape_array.append(data.shape)
        if name:
            name_array.append(name)

    if len(name_array) < len(shape_array):
        param_shape_map = {}
        for i in range(len(param_names)):
            param_shape_map[param_names[i]] = shape_array[i]

        return param_shape_map

    for name in param_names:
        if not name in param_shape_map:
            print("Input {} does not exist!".format(name))
            sys.exit()

    return param_shape_map


def run_one_case(model, param_map):
    # convert np array to model argument
    pp = {}
    for key, val in param_map.items():
        pp[key] = migraphx.argument(val)

    # run the model
    model_outputs = model.run(param_map)

    # convert argument to np array
    outputs = []
    for output in model_outputs:
        outputs.append(np.array(output))

    return outputs


def check_correctness(gold_outputs, outputs, rtol=1e-3, atol=1e-3):
    if len(gold_outputs) != len(outputs):
        print("Number of outputs {} is not equal to expected number {}".format(
            len(outputs), len(gold_outputs)))
        return False

    out_num = len(gold_outputs)
    ret = True
    for i in range(out_num):
        if not np.allclose(gold_outputs[i], outputs[i], rtol, atol):
            print("\nOutput {} is incorrect ...".format(i))
            print("Expected value: \n{}".format(gold_outputs[i]))
            print("......")
            print("Actual value: \n{}\n".format(outputs[i]))
            ret = False

    return ret


def tune_input_shape(model, input_data):
    param_shapes = model.get_parameter_shapes()
    input_shapes = {}
    for name, s in param_shapes.items():
        assert name in input_data
        data_shape = list(input_data[name].shape)
        if not np.array_equal(data_shape, s.lens()):
            input_shapes[name] = data_shape

    return input_shapes


def main():
    args = parse_args()
    test_loc = args.test_dir
    target = args.target

    test_name = os.path.basename(os.path.normpath(test_loc))

    print("Running test \"{}\" on target \"{}\" ...\n".format(
        test_name, target))

    # get model full path
    model_name = get_model_name(test_loc)
    model_path_name = test_loc + '/' + model_name

    # get param names
    param_names = model_parameter_names(model_path_name)

    # get output names
    output_names = model_output_names(model_path_name)

    # get test cases
    cases = get_test_cases(test_loc)
    sample_case = test_loc + '/' + cases[0]
    param_shapes = get_input_shapes(sample_case, param_names)
    for name, dims in param_shapes.items():
        print("Input: {}, shape: {}".format(name, dims))
    print()

    # read and compile model
    model = migraphx.parse_onnx(model_path_name, map_input_dims=param_shapes)
    if args.fp16:
        migraphx.quantize_fp16(model)
    model.compile(migraphx.get_target(target))

    # get test cases
    case_num = len(cases)
    correct_num = 0
    for case_name in cases:
        io_folder = test_loc + '/' + case_name
        input_data = wrapup_inputs(io_folder, param_names)
        gold_outputs = read_outputs(io_folder, output_names)

        # if input shape is different from model shape, reload and recompile
        # model
        input_shapes = tune_input_shape(model, input_data)
        if not len(input_shapes) == 0:
            model = migraphx.parse_onnx(model_path_name,
                                        map_input_dims=input_shapes)
            model.compile(migraphx.get_target(target))

        # run the model and return outputs
        output_data = run_one_case(model, input_data)

        # check output correctness
        ret = check_correctness(gold_outputs,
                                output_data,
                                atol=args.atol,
                                rtol=args.rtol)
        if ret:
            correct_num += 1

        output_str = "PASSED" if ret else "FAILED"
        print("\tCase {}: {}".format(case_name, output_str))

    print("\nTest \"{}\" has {} cases:".format(test_name, case_num))
    print("\t Passed: {}".format(correct_num))
    print("\t Failed: {}".format(case_num - correct_num))
    if case_num > correct_num:
        error_num = case_num - correct_num
        print(str(error_num) + " cases failed!")
        sys.exit(1)

if __name__ == "__main__":
    main()



#####################################################################################
# The MIT License (MIT)
#
# Copyright (c) 2015-2025 Advanced Micro Devices, Inc. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#####################################################################################
import os
import shutil
import argparse
import subprocess
from git_tools import get_changed_files, get_merge_base, get_top, run

CLANG_FORMAT_PATH = '/opt/rocm/llvm/bin'

EXCLUDE_FILES = ['requirements.in', 'onnx.proto']

CLANG_EXTENSIONS = ('.c', '.cpp', '.hpp', '.h', '.cl', '.hip', '.in')
YAPF_EXTENSIONS = ('.py')


def is_excluded(f):
    base = os.path.basename(f)
    return base in EXCLUDE_FILES


def clang_format(against, apply=False, path=CLANG_FORMAT_PATH):
    base = get_merge_base(against)
    clang_format = os.path.join(path, 'clang-format')
    if not os.path.exists(clang_format):
        print(f"{clang_format} not installed. Skipping format.")
        return
    git_clang_format = os.path.join(path, 'git-clang-format')
    if not os.path.exists(git_clang_format):
        print(f"{git_clang_format} not installed. Skipping format.")
        return
    diff_flag = [] if apply else ["--diff"]
    files = get_changed_files(base)
    files = [
        f for f in files if f.endswith(CLANG_EXTENSIONS) and not is_excluded(f)
    ]
    run([git_clang_format, '--binary', clang_format] + diff_flag + [base] +
        files,
        cwd=get_top(),
        verbose=True)


def yapf_format(against, apply=False):
    if not shutil.which('yapf'):
        print("yapf not installed. Skipping format.")
        return
    diff_flag = "--in-place" if apply else "--diff"
    files = ' '.join(get_changed_files(against))
    files = [
        f for f in files if f.endswith(YAPF_EXTENSIONS) and not is_excluded(f)
    ]
    if files:
        run(f"yapf {diff_flag} -p {files}", cwd=get_top(), verbose=True)
    else:
        print("No modified python files to format")


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('against', default='develop', nargs='?')
    parser.add_argument('-i', '--in-place', action='store_true')
    parser.add_argument('-q', '--quiet', action='store_true')
    args = parser.parse_args()
    try:
        clang_format(args.against, apply=args.in_place)
        yapf_format(args.against, apply=args.in_place)
    except subprocess.CalledProcessError as ex:
        if ex.stdout:
            print(ex.stdout)
        if ex.stderr:
            print(ex.stderr)
        if not args.quiet:
            print(f"Command '{ex.cmd}' returned {ex.returncode}")
            raise
            # sys.exit(ex.returncode)


if __name__ == "__main__":
    main()



#####################################################################################
# The MIT License (MIT)
#
# Copyright (c) 2015-2024 Advanced Micro Devices, Inc. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
#####################################################################################
from .base import BaseModel, DecoderModel, OptimumHFModelDownloadMixin
import numpy as np
from transformers import AutoFeatureExtractor


class AutoFeatureExtractorHFMixin(object):

    _processor = None

    @property
    def processor(self):
        if self._processor is None:
            self._processor = AutoFeatureExtractor.from_pretrained(
                self.model_id)
        return self._processor

    def preprocess(self, audio_data, sampling_rate):
        return self.processor(audio_data,
                              sampling_rate=sampling_rate,
                              return_tensors="np")


class Wav2Vec2_base_960h(OptimumHFModelDownloadMixin,
                         AutoFeatureExtractorHFMixin, BaseModel):
    @property
    def model_id(self):
        return "facebook/wav2vec2-base-960h"

    @staticmethod
    def name():
        return "wav2vec2-base-960h"


class WhisperSmallEn(OptimumHFModelDownloadMixin, AutoFeatureExtractorHFMixin,
                     DecoderModel):
    def __init__(self):
        # Whisper specific part
        # TODO get these from config
        self.eos_token_id = 50256  # "<|endoftext|>"
        decoder_start_token_id = 50257  # <|startoftranscript|>
        notimestamps = 50362  # <|notimestamps|>
        sot = [decoder_start_token_id, notimestamps]
        max_length = 448
        self.initial_input_ids = np.array(
            [sot + [self.eos_token_id] * (max_length - len(sot))])

    @property
    def model_id(self):
        return "openai/whisper-small.en"

    @staticmethod
    def name():
        return "whisper-small-en"

    def preprocess(self, *args, **kwargs):
        # result only contains encoder data, extend it with decoder
        result = super().preprocess(*args, **kwargs)
        result["decoder_input_ids"] = np.copy(self.initial_input_ids)
        return result

    def decode_step(self, input_map, output_map):
        timestep = np.argmax(
            input_map["decoder_input_ids"][0] == self.eos_token_id)
        new_token = np.argmax(output_map["logits"][0][timestep - 1])
        input_map["decoder_input_ids"][0][timestep] = new_token
        return new_token == self.eos_token_id



#####################################################################################
# The MIT License (MIT)
#
# Copyright (c) 2015-2024 Advanced Micro Devices, Inc. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
#####################################################################################
from argparse import ArgumentParser
from collections import namedtuple
import warnings

warnings.filterwarnings('ignore')

# datasets
from sample_generator.dataset.imagenet import ImageNet2012Val
from sample_generator.dataset.coco import COCO2017Val
from sample_generator.dataset.librispeech import LibriSpeechASR
from sample_generator.dataset.squad import SQuAD_HF
from sample_generator.dataset.prompts import StylePrompts

# models
from sample_generator.model.image import ResNet50_v1, ResNet50_v1_5, VitBasePatch16_224, TIMM_MobileNetv3_large
from sample_generator.model.text import BERT_large_uncased, DistilBERT_base_cased_distilled_SQuAD, RobertaBaseSquad2
from sample_generator.model.text import GPTJ, Llama2_7b_chat_hf, Llama3_8b_instruct, T5_base, Gemma_2b_it
from sample_generator.model.audio import Wav2Vec2_base_960h, WhisperSmallEn
from sample_generator.model.hybrid import ClipVitLargePatch14
from sample_generator.model.diffusion import StableDiffusion21, StableDiffusionXL

# generator
from sample_generator.generator import generate_test_dataset, generate_diffusion_data

DatasetModelsPair = namedtuple('DatasetModelsPair', ['dataset', 'models'])

imagenet_models = (
    ResNet50_v1,
    ResNet50_v1_5,
    VitBasePatch16_224,
    TIMM_MobileNetv3_large,
    ClipVitLargePatch14,
)

squad_models = (
    BERT_large_uncased,
    DistilBERT_base_cased_distilled_SQuAD,
    RobertaBaseSquad2,
    GPTJ,
    T5_base,
    Gemma_2b_it,
    Llama2_7b_chat_hf,
    Llama3_8b_instruct,
)

librispeech_models = (Wav2Vec2_base_960h, WhisperSmallEn)

diffusion_models = (StableDiffusion21, StableDiffusionXL)

default_dataset_model_mapping = {
    "image": DatasetModelsPair(ImageNet2012Val, imagenet_models),
    "text": DatasetModelsPair(SQuAD_HF, squad_models),
    "audio": DatasetModelsPair(LibriSpeechASR, librispeech_models),
    "diffusion": DatasetModelsPair((COCO2017Val, StylePrompts),
                                   diffusion_models)
}


def get_args():
    parser = ArgumentParser()
    parser.add_argument(
        '--image',
        choices=['all', 'none'] + [model.name() for model in imagenet_models],
        nargs='+',
        default='all',
        dest='image_model_names',
        type=str,
        help=
        f'Image models to test with {ImageNet2012Val.name()} dataset samples')
    parser.add_argument(
        '--text',
        choices=['all', 'none'] + [model.name() for model in squad_models],
        nargs='+',
        default='all',
        dest='text_model_names',
        type=str,
        help=f'Text models to test with {SQuAD_HF.name()} dataset samples')
    parser.add_argument(
        '--audio',
        choices=['all', 'none'] +
        [model.name() for model in librispeech_models],
        nargs='+',
        default='all',
        dest='audio_model_names',
        type=str,
        help=
        f'Audio models to test with {LibriSpeechASR.name()} dataset samples')
    parser.add_argument(
        '--diffusion',
        choices=['all', 'none'] + [model.name() for model in diffusion_models],
        nargs='+',
        default='all',
        dest='diffusion_model_names',
        type=str,
        help=
        f'DiffusionModel models to test with {COCO2017Val.name()} and {StylePrompts} dataset samples'
    )
    parser.add_argument(
        '--output-folder-prefix',
        default='generated',
        help='Output path will be "<this-prefix>/<dataset-name>/<model-name>"')
    parser.add_argument(
        '--sample-limit',
        default=5,
        type=int,
        help="Max number of samples generated. Use 0 to ignore it.")
    parser.add_argument(
        '--decode-limit',
        default=5,
        type=int,
        help=
        "Max number of sum-samples generated for decoder models. Use 0 to ignore it. (Only for decoder models)"
    )
    return parser.parse_args()


def get_dataset_models_pairs(dataset_type, model_names):
    if 'none' in model_names:
        return None
    ds_ms_mapping = default_dataset_model_mapping[dataset_type]
    if 'all' in model_names:
        return ds_ms_mapping

    return DatasetModelsPair(dataset=ds_ms_mapping.dataset,
                             models=(model for model in ds_ms_mapping.models
                                     if model.name() in model_names))


def main(image_model_names='all',
         text_model_names='all',
         audio_model_names='all',
         diffusion_model_names='all',
         output_folder_prefix='generated',
         sample_limit=5,
         decode_limit=5):
    for dataset_type, model_names in zip(
        ('image', 'text', 'audio'),
        (image_model_names, text_model_names, audio_model_names)):
        dataset_model_pair = get_dataset_models_pairs(dataset_type,
                                                      model_names)
        if dataset_model_pair is None:
            print(f"Skip {dataset_type}...")
            continue

        dataset = dataset_model_pair.dataset
        for model in dataset_model_pair.models:
            try:
                generate_test_dataset(
                    model(),
                    dataset(),
                    output_folder_prefix=output_folder_prefix,
                    sample_limit=sample_limit,
                    decode_limit=decode_limit)
            except Exception as e:
                print(f"Something went wrong:\n{e}\nSkipping model...")
                continue

    for dataset_type, model_names in zip(('diffusion', ),
                                         (diffusion_model_names)):
        dataset_model_pair = get_dataset_models_pairs(dataset_type,
                                                      model_names)
        if dataset_model_pair is None:
            print(f"Skip {dataset_type}...")
            continue

        image_dataset, prompt_dataset = dataset_model_pair.dataset
        for model in dataset_model_pair.models:
            try:
                generate_diffusion_data(
                    model(),
                    image_dataset(),
                    prompt_dataset(),
                    output_folder_prefix=output_folder_prefix,
                    sample_limit=sample_limit,
                    decode_limit=decode_limit)
            except Exception as e:
                print(f"Something went wrong:\n{e}\nSkipping model...")
                continue

    print(
        f'Use this to test MIGraphX with the result:\n./test_models.sh {output_folder_prefix}'
    )


if '__main__' == __name__:
    args = get_args()
    main(**vars(args))



#####################################################################################
# The MIT License (MIT)
#
# Copyright (c) 2015-2022 Advanced Micro Devices, Inc. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#####################################################################################
import migraphx, array, sys


def test_conv_relu():
    p = migraphx.parse_onnx("conv_relu_maxpool_test.onnx")
    print(p)
    s1 = p.get_output_shapes()[-1]
    print("Compiling ...")
    p.compile(migraphx.get_target("ref"))
    print(p)
    s2 = p.get_output_shapes()[-1]
    assert s1 == s2
    params = {}

    for key, value in p.get_parameter_shapes().items():
        print("Parameter {} -> {}".format(key, value))
        params[key] = migraphx.generate_argument(value)

    r = p.run(params)[-1]
    print(r)


def create_buffer(t, data, shape):
    a = array.array(t, data)
    if sys.version_info >= (3, 0):
        m = memoryview(a.tobytes())
        return m.cast(t, shape)
    else:
        m = memoryview(a.tostring())
        return m


def test_add_scalar():
    p = migraphx.parse_onnx("add_scalar_test.onnx")
    print(p)
    s1 = p.get_output_shapes()[-1]
    print("Compiling ...")
    p.compile(migraphx.get_target("ref"))
    print(p)
    s2 = p.get_output_shapes()[-1]
    assert s1 == s2

    d0 = list(range(120))
    arg0 = create_buffer("B", d0, [2, 3, 4, 5])
    d1 = [1]
    arg1 = create_buffer("B", d1, ())

    params = {}
    params["0"] = migraphx.argument(arg0)
    params["1"] = migraphx.argument(arg1)

    r = p.run(params)[-1]
    print(r)


def test_module():
    p = migraphx.parse_onnx("add_scalar_test.onnx")
    mm = p.get_main_module()
    print(p)
    print(mm)


test_conv_relu()
test_module()
if sys.version_info >= (3, 0):
    test_add_scalar()



#####################################################################################
# The MIT License (MIT)
#
# Copyright (c) 2015-2024 Advanced Micro Devices, Inc. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#####################################################################################

from optimum.exporters.onnx import main_export
from optimum.exporters.onnx.model_configs import WhisperOnnxConfig
from transformers import AutoConfig

from optimum.exporters.onnx.base import ConfigBehavior
from typing import Dict


class CustomWhisperOnnxConfig(WhisperOnnxConfig):
    @property
    def inputs(self) -> Dict[str, Dict[int, str]]:
        common_inputs = {}

        if self._behavior is ConfigBehavior.ENCODER:
            common_inputs["input_features"] = {
                0: "batch_size",
                1: "feature_size",
                2: "encoder_sequence_length"
            }

        if self._behavior is ConfigBehavior.DECODER:
            common_inputs["decoder_input_ids"] = {
                0: "batch_size",
                1: "decoder_sequence_length"
            }
            common_inputs["decoder_attention_mask"] = {
                0: "batch_size",
                1: "decoder_sequence_length"
            }
            common_inputs["encoder_outputs"] = {
                0: "batch_size",
                1: "encoder_sequence_length"
            }

        return common_inputs

    @property
    def torch_to_onnx_input_map(self) -> Dict[str, str]:
        if self._behavior is ConfigBehavior.DECODER:
            return {
                "decoder_input_ids": "input_ids",
                "decoder_attention_mask": "attention_mask",
                "encoder_outputs": "encoder_hidden_states",
            }
        return {}


def export():
    model_id = "openai/whisper-tiny.en"
    config = AutoConfig.from_pretrained(model_id)

    custom_whisper_onnx_config = CustomWhisperOnnxConfig(
        config=config,
        task="automatic-speech-recognition",
    )

    encoder_config = custom_whisper_onnx_config.with_behavior("encoder")
    decoder_config = custom_whisper_onnx_config.with_behavior("decoder",
                                                              use_past=False)

    custom_onnx_configs = {
        "encoder_model": encoder_config,
        "decoder_model": decoder_config,
    }

    output = "models/whisper-tiny.en_modified"
    main_export(model_id,
                output=output,
                no_post_process=True,
                do_validation=False,
                custom_onnx_configs=custom_onnx_configs)

    print(f"Done. Check {output}")


if __name__ == "__main__":
    export()



#####################################################################################
# The MIT License (MIT)
#
# Copyright (c) 2015-2025 Advanced Micro Devices, Inc. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#####################################################################################
import string
import sys
import re
import runpy
from functools import wraps
from typing import Any, Callable, Dict, List, Optional, Tuple, Union
from pathlib import Path

type_map: Dict[str, Callable[['Parameter'], None]] = {}
cpp_type_map: Dict[str, str] = {}
functions: List['Function'] = []
cpp_classes: List['CPPClass'] = []
error_type = ''
success_type = ''
try_wrap = ''

export_c_macro = 'MIGRAPHX_C_EXPORT'

c_header_preamble: List[str] = []
c_api_body_preamble: List[str] = []
cpp_header_preamble: List[str] = []


def bad_param_error(msg: str):
    return 'throw std::runtime_error("{}")'.format(msg)


class Template(string.Template):
    idpattern = '[_a-zA-Z0-9@]+'


class Type:
    def __init__(self, name: str) -> None:
        self.name = name.strip()

    def is_pointer(self) -> bool:
        return self.name.endswith('*')

    def is_reference(self) -> bool:
        return self.name.endswith('&')

    def is_const(self) -> bool:
        return self.name.startswith('const ')

    def is_variadic(self):
        return self.name.startswith('...')

    def add_pointer(self) -> 'Type':
        return Type(self.name + '*')

    def add_reference(self):
        return Type(self.name + '&')

    def add_const(self) -> 'Type':
        return Type('const ' + self.name)

    def inner_type(self) -> Optional['Type']:
        i = self.name.find('<')
        j = self.name.rfind('>')
        if i > 0 and j > 0:
            return Type(self.name[i + 1:j])
        else:
            return None

    def remove_generic(self) -> 'Type':
        i = self.name.find('<')
        j = self.name.rfind('>')
        if i > 0 and j > 0:
            return Type(self.name[0:i] + self.name[j + 1:])
        else:
            return self

    def remove_pointer(self) -> 'Type':
        if self.is_pointer():
            return Type(self.name[0:-1])
        return self

    def remove_reference(self) -> 'Type':
        if self.is_reference():
            return Type(self.name[0:-1])
        return self

    def remove_const(self) -> 'Type':
        if self.is_const():
            return Type(self.name[6:])
        return self

    def basic(self) -> 'Type':
        return self.remove_pointer().remove_const().remove_reference()

    def decay(self) -> 'Type':
        t = self.remove_reference()
        if t.is_pointer():
            return t
        else:
            return t.remove_const()

    def const_compatible(self, t: 'Type'):
        if t.is_const():
            return self.add_const()
        return self

    def str(self) -> str:
        return self.name


header_function = Template('''
${export_c_macro} ${error_type} ${name}(${params});
''')

function_pointer_typedef = Template('''
typedef ${error_type} (*${fname})(${params});
''')

c_api_impl = Template('''
extern "C" ${error_type} ${name}(${params})
{
    ${va_start}auto api_error_result = ${try_wrap}([&] {
        ${body};
    });
    ${va_end}return api_error_result;
}
''')


class CFunction:
    def __init__(self, name: str) -> None:
        self.name = name
        self.params: List[str] = []
        self.body: List[str] = []
        self.va_start: List[str] = []
        self.va_end: List[str] = []

    def add_param(self, type: str, pname: str) -> None:
        self.params.append('{} {}'.format(type, pname))

    def add_statement(self, stmt: str) -> None:
        self.body.append(stmt)

    def add_vlist(self, name: str) -> None:
        last_param = self.params[-1].split()[-1]
        self.va_start = [
            'va_list {};'.format(name),
            'va_start({}, {});'.format(name, last_param)
        ]
        self.va_end = ['va_end({});'.format(name)]
        self.add_param('...', '')

    def substitute(self, form: Template, **kwargs) -> str:
        return form.substitute(error_type=error_type,
                               try_wrap=try_wrap,
                               name=self.name,
                               params=', '.join(self.params),
                               body=";\n        ".join(self.body),
                               va_start="\n    ".join(self.va_start),
                               va_end="\n    ".join(self.va_end),
                               **kwargs)

    def generate_header(self) -> str:
        return self.substitute(header_function, export_c_macro=export_c_macro)

    def generate_function_pointer(self, name: Optional[str] = None) -> str:
        return self.substitute(function_pointer_typedef,
                               fname=name or self.name)

    def generate_body(self) -> str:
        return self.substitute(c_api_impl)


class BadParam:
    def __init__(self, cond: str, msg: str) -> None:
        self.cond = cond
        self.msg = msg


class Parameter:
    def __init__(self,
                 name: str,
                 type: str,
                 optional: bool = False,
                 returns: bool = False,
                 virtual: bool = False,
                 this: bool = False,
                 hidden: bool = False) -> None:
        self.name = name
        self.type = Type(type)
        self.optional = optional
        self.cparams: List[Tuple[str, str]] = []
        self.size_cparam = -1
        self.size_name = ''
        self.read = '${name}'
        self.write = ['*${name} = ${result}']
        self.cpp_read = '${name}'
        self.cpp_write = '${name}'
        self.returns = returns
        self.virtual = virtual
        self.this = this
        self.hidden = hidden
        self.bad_param_check: Optional[BadParam] = None
        self.virtual_read: Optional[List[str]] = None
        self.virtual_write: Optional[str] = None

    def get_name(self, prefix: Optional[str] = None) -> str:
        if prefix:
            return prefix + self.name
        else:
            return self.name

    def get_cpp_type(self) -> str:
        if self.type.str() in cpp_type_map:
            return cpp_type_map[self.type.basic().str()]
        elif self.type.basic().str() in cpp_type_map:
            return cpp_type_map[self.type.basic().str()]
        elif self.returns:
            return self.type.decay().str()
        else:
            return self.type.str()

    def substitute(self,
                   s: str,
                   prefix: Optional[str] = None,
                   result: Optional[str] = None) -> str:
        ctype = None
        if len(self.cparams) > 0:
            ctype = Type(self.cparams[0][0]).basic().str()
        return Template(s).safe_substitute(name=self.get_name(prefix),
                                           type=self.type.str(),
                                           ctype=ctype or '',
                                           cpptype=self.get_cpp_type(),
                                           size=self.size_name,
                                           result=result or '')

    def add_param(self, t: Union[str, Type],
                  name: Optional[str] = None) -> None:
        if not isinstance(t, str):
            t = t.str()
        self.cparams.append((t, name or self.name))

    def add_size_param(self, name: Optional[str] = None) -> None:
        self.size_cparam = len(self.cparams)
        self.size_name = name or self.name + '_size'
        if self.returns:
            self.add_param('size_t *', self.size_name)
        else:
            self.add_param('size_t', self.size_name)

    def bad_param(self, cond: str, msg: str) -> None:
        self.bad_param_check = BadParam(cond, msg)

    def remove_size_param(self, name):
        p = None
        if self.size_cparam >= 0:
            p = self.cparams[self.size_cparam]
            del self.cparams[self.size_cparam]
            self.size_name = name
        return p

    def update(self) -> None:
        t = self.type.basic().str()
        g = self.type.remove_generic().basic().str()
        if t in type_map:
            type_map[t](self)
        elif g in type_map:
            type_map[g](self)
        else:
            if self.returns:
                self.add_param(self.type.remove_reference().add_pointer())
            else:
                self.add_param(self.type.remove_reference())
        if isinstance(self.write, str):
            raise ValueError("Error for {}: write cannot be a string".format(
                self.type.str()))

    def virtual_arg(self, prefix: Optional[str] = None) -> List[str]:
        read = self.virtual_read
        if not read and len(self.write) >= len(self.cparams):
            read = [
                Template(w.partition('=')[2]).safe_substitute(result='${name}')
                for w in self.write
            ]
        if not read:
            raise ValueError("No virtual_read parameter provided for: " +
                             self.type.str())
        if isinstance(read, str):
            raise ValueError(
                "Error for {}: virtual_read cannot be a string".format(
                    self.type.str()))
        return [self.substitute(r, prefix=prefix) for r in read]

    def virtual_param(self, prefix: Optional[str] = None) -> str:
        return self.substitute('${type} ${name}', prefix=prefix)

    def virtual_output_args(self, prefix: Optional[str] = None) -> List[str]:
        container_type = self.type.remove_generic().basic().str()
        decl_list: List[str] = []
        container = (container_type == "std::vector"
                     or container_type == "vector")
        for t, n, in self.cparams:
            if not decl_list and container:
                decl_list.append('{prefix}{n}.data()'.format(prefix=prefix
                                                             or '',
                                                             n=n))
            else:
                decl_list.append('&{prefix}{n}'.format(prefix=prefix or '',
                                                       n=n))
        return decl_list

    def virtual_output_declarations(self,
                                    prefix: Optional[str] = None) -> List[str]:
        container_type = self.type.remove_generic().basic().str()
        container = (container_type == "std::vector"
                     or container_type == "vector")
        decl_list: List[str] = []
        for t, n, in self.cparams:
            if not decl_list and container:
                inner_t = self.type.inner_type()
                if inner_t:
                    decl_list.append(
                        'std::array<{inner_t}, 1024> {prefix}{n};'.format(
                            inner_t=inner_t.str(), prefix=prefix or '', n=n))
            else:
                decl_list.append(
                    'std::remove_pointer_t<{type}> {prefix}{n}'.format(
                        type=Type(t).str(), prefix=prefix or '', n=n))
                decl_list[-1] += '=1024;' if container else ';'
        return decl_list

    def virtual_output(self, prefix: Optional[str] = None) -> str:
        write = self.virtual_write
        if not write:
            if '*' in self.read or '->' in self.read:
                write = Template(self.read).safe_substitute(name='(&${name})')
            else:
                write = self.read
        return self.substitute(write, prefix=prefix)

    def cpp_param(self, prefix: Optional[str] = None) -> str:
        return self.substitute('${cpptype} ${name}', prefix=prefix)

    def cpp_arg(self, prefix: Optional[str] = None) -> str:
        return self.substitute(self.cpp_read, prefix=prefix)

    def cpp_output_args(self, prefix: Optional[str] = None) -> List[str]:
        return [
            '&{prefix}{n}'.format(prefix=prefix, n=n) for t, n in self.cparams
        ]

    def output_declarations(self, prefix: Optional[str] = None) -> List[str]:
        return [
            '{type} {prefix}{n};'.format(type=Type(t).remove_pointer().str(),
                                         prefix=prefix,
                                         n=n) for t, n in self.cparams
        ]

    def output_args(self, prefix=None):
        return [
            '&{prefix}{n};'.format(prefix=prefix, n=n) for t, n in self.cparams
        ]

    def cpp_output(self, prefix: Optional[str] = None) -> str:
        return self.substitute(self.cpp_write, prefix=prefix)

    def input(self, prefix: Optional[str] = None) -> str:
        return '(' + self.substitute(self.read, prefix=prefix) + ')'

    def outputs(self, result: Optional[str] = None) -> List[str]:
        return [self.substitute(w, result=result) for w in self.write]

    def add_to_cfunction(self, cfunction: CFunction) -> None:
        for t, name in self.cparams:
            if t.startswith('...'):
                cfunction.add_vlist(name)
            else:
                cfunction.add_param(self.substitute(t), self.substitute(name))
        if self.bad_param_check:
            msg = 'Bad parameter {name}: {msg}'.format(
                name=self.name, msg=self.bad_param_check.msg)
            cfunction.add_statement('if ({cond}) {body}'.format(
                cond=self.substitute(self.bad_param_check.cond),
                body=bad_param_error(msg)))


def template_var(s: str) -> str:
    return '${' + s + '}'


def to_template_vars(params: List[Union[Any, Parameter]]) -> str:
    return ', '.join([template_var(p.name) for p in params])


class Function:
    def __init__(self,
                 name: str,
                 params: Optional[List[Parameter]] = None,
                 shared_size: bool = False,
                 returns: Optional[str] = None,
                 invoke: Optional[str] = None,
                 fname: Optional[str] = None,
                 return_name: Optional[str] = None,
                 virtual: bool = False,
                 **kwargs) -> None:
        self.name = name
        self.params = params or []
        self.shared_size = False
        self.cfunction: Optional[CFunction] = None
        self.fname = fname
        self.invoke = invoke or '${__fname__}($@)'
        self.return_name = return_name or 'out'
        self.returns = Parameter(self.return_name, returns,
                                 returns=True) if returns else None
        for p in self.params:
            p.virtual = virtual
        if self.returns:
            self.returns.virtual = virtual

    def share_params(self) -> None:
        if self.shared_size == True:
            size_param_name = 'size'
            size_type = Type('size_t')
            for param in self.params:
                p = param.remove_size_param(size_param_name)
                if p:
                    size_type = Type(p[0])
            self.params.append(Parameter(size_param_name, size_type.str()))

    def update(self) -> None:
        self.share_params()
        for param in self.params:
            param.update()
        if self.returns:
            self.returns.update()
        self.create_cfunction()

    def inputs(self) -> str:
        return ', '.join([p.input() for p in self.params])

    # TODO: Shoule we remove Optional?
    def input_map(self) -> Dict[str, Optional[str]]:
        m: Dict[str, Optional[str]] = {}
        for p in self.params:
            m[p.name] = p.input()
        m['return'] = self.return_name
        m['@'] = self.inputs()
        m['__fname__'] = self.fname
        return m

    def get_invoke(self) -> str:
        return Template(self.invoke).safe_substitute(self.input_map())

    def write_to_tmp_var(self) -> bool:
        if not self.returns:
            return False
        return len(self.returns.write) > 1 or self.returns.write[0].count(
            '${result}') > 1

    def get_cfunction(self) -> CFunction:
        if self.cfunction:
            return self.cfunction
        raise Exception(
            "self.cfunction is None: self.update() needs to be called.")

    def create_cfunction(self) -> None:
        self.cfunction = CFunction(self.name)
        # Add the return as a parameter
        if self.returns:
            self.returns.add_to_cfunction(self.cfunction)
        # Add the input parameters
        for param in self.params:
            param.add_to_cfunction(self.cfunction)
        f: Optional[str] = self.get_invoke()
        # Write the assignments
        assigns = []
        if self.returns:
            result = f
            if self.write_to_tmp_var() and f:
                f = 'auto&& api_result = ' + f
                result = 'api_result'
            else:
                f = None
            assigns = self.returns.outputs(result)
        if f:
            self.cfunction.add_statement(f)
        for assign in assigns:
            self.cfunction.add_statement(assign)


cpp_class_template = Template('''

struct ${name} : handle_base<${ctype}, decltype(&${destroy}), ${destroy}>
{
    ${name}(${ctype} p, bool own = true)
    : m_handle(nullptr)
    {
        this->set_handle(p, own);
    }
    ${constructors}

    ${methods}
};
''')

cpp_class_method_template = Template('''
    ${return_type} ${name}(${params}) const
    {
        ${outputs}
        this->call_handle(${args});
        return ${result};
    }
''')

cpp_class_void_method_template = Template('''
    void ${name}(${params}) const
    {
        this->call_handle(${args});
    }
''')

cpp_class_constructor_template = Template('''
    ${name}(${params})
    : m_handle(nullptr)
    {
        m_handle = this->make_handle(${args});
    }
''')


class CPPMember:
    def __init__(self,
                 name: str,
                 function: Function,
                 prefix: str,
                 method: bool = True) -> None:
        self.name = name
        self.function = function
        self.prefix = prefix
        self.method = method

    def get_function_params(self) -> List[Union[Any, Parameter]]:
        if self.method:
            return self.function.params[1:]
        else:
            return self.function.params

    def get_args(self) -> str:
        output_args = []
        if self.function.returns:
            output_args = self.function.returns.cpp_output_args(self.prefix)
        if not self.function.cfunction:
            raise Exception('self.function.update() must be called')
        return ', '.join(
            ['&{}'.format(self.function.cfunction.name)] + output_args +
            [p.cpp_arg(self.prefix) for p in self.get_function_params()])

    def get_params(self) -> str:
        return ', '.join(
            [p.cpp_param(self.prefix) for p in self.get_function_params()])

    def get_return_declarations(self) -> str:
        if self.function.returns:
            return '\n        '.join([
                d
                for d in self.function.returns.output_declarations(self.prefix)
            ])
        else:
            return ''

    def get_result(self):
        return self.function.returns.input(self.prefix)

    def generate_method(self) -> str:
        if not self.function.cfunction:
            raise Exception('self.function.update() must be called')
        if self.function.returns:
            return_type = self.function.returns.get_cpp_type()
            return cpp_class_method_template.safe_substitute(
                return_type=return_type,
                name=self.name,
                cfunction=self.function.cfunction.name,
                result=self.function.returns.cpp_output(self.prefix),
                params=self.get_params(),
                outputs=self.get_return_declarations(),
                args=self.get_args(),
                success=success_type)
        else:
            return cpp_class_void_method_template.safe_substitute(
                name=self.name,
                cfunction=self.function.cfunction.name,
                params=self.get_params(),
                args=self.get_args(),
                success=success_type)

    def generate_constructor(self, name: str) -> str:
        if not self.function.cfunction:
            raise Exception('self.function.update() must be called')
        return cpp_class_constructor_template.safe_substitute(
            name=name,
            cfunction=self.function.cfunction.name,
            params=self.get_params(),
            args=self.get_args(),
            success=success_type)


class CPPClass:
    def __init__(self, name: str, ctype: str) -> None:
        self.name = name
        self.ctype = ctype
        self.constructors: List[CPPMember] = []
        self.methods: List[CPPMember] = []
        self.prefix = 'p'

    def add_method(self, name: str, f: Function) -> None:
        self.methods.append(CPPMember(name, f, self.prefix, method=True))

    def add_constructor(self, name: str, f: Function) -> None:
        self.constructors.append(CPPMember(name, f, self.prefix, method=True))

    def generate_methods(self) -> str:
        return '\n    '.join([m.generate_method() for m in self.methods])

    def generate_constructors(self) -> str:
        return '\n    '.join(
            [m.generate_constructor(self.name) for m in self.constructors])

    def substitute(self, s: Union[string.Template, str], **kwargs) -> str:
        t = string.Template(s) if isinstance(s, str) else s
        destroy = self.ctype + '_destroy'
        return t.safe_substitute(name=self.name,
                                 ctype=self.ctype,
                                 destroy=destroy,
                                 **kwargs)

    def generate(self) -> str:
        return self.substitute(
            cpp_class_template,
            constructors=self.substitute(self.generate_constructors()),
            methods=self.substitute(self.generate_methods()))


def params(virtual: Optional[Dict[str, str]] = None,
           **kwargs) -> List[Parameter]:
    result = []
    v: Dict[str, str] = virtual or {}
    for name in v:
        result.append(Parameter(name, v[name]))
    for name in kwargs:
        result.append(Parameter(name, kwargs[name]))
    return result


gparams = params


def add_function(name: str, *args, **kwargs) -> Function:
    f = Function(name, *args, **kwargs)
    functions.append(f)
    return f


def once(f: Callable) -> Any:
    @wraps(f)
    def decorated(*args, **kwargs):
        if not decorated.has_run:
            decorated.has_run = True
            return f(*args, **kwargs)

    d: Any = decorated
    d.has_run = False
    return d


@once
def process_functions() -> None:
    for f in functions:
        f.update()


def generate_lines(p: List[str]) -> str:
    return '\n'.join(p)


def generate_c_header() -> str:
    process_functions()
    return generate_lines(
        c_header_preamble +
        [f.get_cfunction().generate_header() for f in functions])


def generate_c_api_body() -> str:
    process_functions()
    return generate_lines(
        c_api_body_preamble +
        [f.get_cfunction().generate_body() for f in functions])


def generate_cpp_header() -> str:
    process_functions()
    return generate_lines(cpp_header_preamble +
                          [c.generate() for c in cpp_classes])


c_type_map: Dict[str, Type] = {}


def cwrap(name: str, c_type: Optional[str] = None) -> Callable:
    def with_cwrap(f):
        type_map[name] = f
        if c_type:
            c_type_map[name] = Type(c_type)

        @wraps(f)
        def decorated(*args, **kwargs):
            return f(*args, **kwargs)

        return decorated

    return with_cwrap


handle_typedef = Template('''
typedef struct ${ctype} * ${ctype}_t;
typedef const struct ${ctype} * const_${ctype}_t;
''')

handle_definition = Template('''
extern "C" struct ${ctype};
struct ${ctype} {
    template<class... Ts>
    ${ctype}(Ts&&... xs)
    : object(std::forward<Ts>(xs)...) // NOLINT(readability-redundant-member-init)
    {}
    ${cpptype} object;
};
''')

handle_preamble = '''
template<class T, class U, class Target=std::remove_pointer_t<T>>
static Target* object_cast(U* x)
{
    return reinterpret_cast<Target*>(x);
}
template<class T, class U, class Target=std::remove_pointer_t<T>>
static const Target* object_cast(const U* x)
{
    return reinterpret_cast<const Target*>(x);
}

template <class T, class... Ts, class Target = std::remove_pointer_t<T>>
static Target* allocate(Ts&&... xs)
{
    if constexpr(std::is_aggregate<Target>{})
        return new Target{std::forward<Ts>(xs)...}; // NOLINT
    else
        return new Target(std::forward<Ts>(xs)...); // NOLINT
}

template<class T>
static void destroy(T* x)
{
    delete x; // NOLINT
}


// TODO: Move to interface preamble
template <class C, class D>
struct manage_generic_ptr
{
    manage_generic_ptr() = default;
    
    manage_generic_ptr(std::nullptr_t)
    {
    }

    manage_generic_ptr(void* pdata, const char* obj_tname, C pcopier, D pdeleter)
        : data(nullptr), obj_typename(obj_tname), copier(pcopier), deleter(pdeleter)
    {
        copier(&data, pdata);
    }

    manage_generic_ptr(const manage_generic_ptr& rhs)
        : data(nullptr), obj_typename(rhs.obj_typename), copier(rhs.copier), deleter(rhs.deleter)
    {
        if(copier)
            copier(&data, rhs.data);
    }

    manage_generic_ptr(manage_generic_ptr&& other) noexcept
        : data(other.data), obj_typename(other.obj_typename), copier(other.copier), deleter(other.deleter)
    {
        other.data    = nullptr;
        other.obj_typename = "";
        other.copier  = nullptr;
        other.deleter = nullptr;
    }

    manage_generic_ptr& operator=(manage_generic_ptr rhs)
    {
        std::swap(data, rhs.data);
        std::swap(obj_typename, rhs.obj_typename);
        std::swap(copier, rhs.copier);
        std::swap(deleter, rhs.deleter);
        return *this;
    }

    ~manage_generic_ptr()
    {
        if(data != nullptr)
            deleter(data);
    }

    void* data = nullptr;
    const char* obj_typename = "";
    C copier   = nullptr;
    D deleter  = nullptr;
};
'''

cpp_handle_preamble = '''
template<class T, class Deleter, Deleter deleter>
struct handle_base
{

    template<class F, class... Ts>
    void make_handle(F f, Ts&&... xs)
    {
        T* result = nullptr;
        auto e = F(&result, std::forward<Ts>(xs)...);
        if (e != ${success})
            throw std::runtime_error("Failed to call function");
        set_handle(result);
    }

    template<class F, class... Ts>
    void call_handle(F f, Ts&&... xs)
    {
        auto e = F(this->get_handle_ptr(), std::forward<Ts>(xs)...);
        if (e != ${success})
            throw std::runtime_error("Failed to call function");
    }

    const std::shared_ptr<T>& get_handle() const
    {
        return m_handle;
    }

    T* get_handle_ptr() const
    {
        assert(m_handle != nullptr);
        return get_handle().get();
    }

    void set_handle(T* ptr, bool own = true)
    {
        if (own)
            m_handle = std::shared_ptr<T>{ptr, deleter};
        else
            m_handle = std::shared_ptr<T>{ptr, [](T*) {}};
    }

protected:
    std::shared_ptr<T> m_handle;
};

'''


@once
def add_handle_preamble() -> None:
    c_api_body_preamble.append(handle_preamble)
    cpp_header_preamble.append(
        string.Template(cpp_handle_preamble).substitute(success=success_type))


def add_handle(name: str,
               ctype: str,
               cpptype: str,
               destroy: Optional[str] = None,
               ref=False,
               skip_def=False) -> None:
    opaque_type = ctype + '_t'
    const_opaque_type = 'const_' + opaque_type

    def handle_wrap(p: Parameter):
        t = Type(opaque_type)
        if p.type.is_const():
            t = Type('const_' + opaque_type)
        # p.read = 'object_cast<${ctype}>(&(${name}))'
        if p.virtual:
            p.add_param(t)
        elif p.returns:
            p.add_param(t.add_pointer())
        else:
            p.add_param(t)
            p.bad_param('${name} == nullptr', 'Null pointer')
        if p.type.is_reference():
            p.virtual_read = ['object_cast<${ctype}>(&(${name}))']
            p.cpp_write = '${cpptype}(${name}, false)'
            p.write = ['*${name} = object_cast<${ctype}>(&(${result}))']
        elif p.type.is_pointer():
            p.virtual_read = ['object_cast<${ctype}>(${result})']
            p.cpp_write = '${cpptype}(${name}, false)'
            p.write = ['*${name} = object_cast<${ctype}>(${result})']
        else:
            p.virtual_read = ['object_cast<${ctype}>(&(${name}))']
            p.cpp_write = '${cpptype}(${name})'
            p.write = ['*${name} = allocate<${ctype}>(${result})']
        if skip_def:
            p.read = '*${name}'
        else:
            p.read = '${name}->object'
        p.cpp_read = '${name}.get_handle_ptr()'

    type_map[cpptype] = handle_wrap
    if not ref:
        add_function(destroy or ctype + '_' + 'destroy',
                     params({name: opaque_type}),
                     fname='destroy')
        add_function(ctype + '_' + 'assign_to',
                     params(output=opaque_type, input=const_opaque_type),
                     invoke='*output = *input')
    add_handle_preamble()
    c_header_preamble.append(handle_typedef.substitute(locals()))
    if not skip_def:
        c_api_body_preamble.append(handle_definition.substitute(locals()))


@cwrap('std::vector')
def vector_c_wrap(p: Parameter) -> None:
    inner = p.type.inner_type()
    # Not a generic type
    if not inner:
        return
    if inner.str() in c_type_map:
        inner = c_type_map[inner.str()]

    t = inner.add_pointer()
    if p.type.is_reference():
        if p.type.is_const():
            t = t.add_const()
    if p.returns:
        if p.type.is_reference():
            p.add_param(t.add_pointer())
            p.add_size_param()
            p.bad_param('${name} == nullptr or ${size} == nullptr',
                        'Null pointer')
        elif p.virtual:
            p.add_param(t)
            p.add_size_param()
            p.bad_param('${name} == nullptr or ${size} == nullptr',
                        'Null pointer')
            p.virtual_write = '{${name}.begin(), ${name}.begin()+${size}}; // cppcheck-suppress returnDanglingLifetime'
        else:
            p.add_param(t)
            p.bad_param('${name} == nullptr', 'Null pointer')
    else:
        p.add_param(t)
        p.add_size_param()
        p.bad_param('${name} == nullptr and ${size} != 0', 'Null pointer')

    p.read = '${type}(${name}, ${name}+${size})'
    p.cpp_write = '${type}(${name}, ${name}+${size})'
    p.virtual_read = ['${name}.data()', '${name}.size()']
    if p.type.is_reference():
        p.write = [
            '*${name} = ${result}.data()', '*${size} = ${result}.size()'
        ]
    else:
        p.write = ['std::copy(${result}.begin(), ${result}.end(), ${name})']


@cwrap('std::string', 'char*')
def string_c_wrap(p: Parameter) -> None:
    t = Type('char*')
    if p.returns:
        if p.type.is_reference():
            p.add_param(t.add_pointer())
            p.bad_param('${name} == nullptr', 'Null pointer')
        else:
            p.add_param(t)
            p.add_param('size_t', p.name + '_size')
            p.bad_param('${name} == nullptr', 'Null pointer')
    else:
        p.add_param(t)
        p.bad_param('${name} == nullptr', 'Null pointer')

    p.read = '${type}(${name})'
    p.cpp_write = '${type}(${name})'
    p.virtual_read = ['${name}.c_str()']
    if p.type.is_reference():
        p.write = ['*${name} = ${result}.c_str()']
    else:
        p.write = [
            'auto* it = std::copy_n(${result}.begin(), std::min(${result}.size(), ${name}_size - 1), ${name});'
            '*it = \'\\0\''
        ]


class Handle:
    def __init__(self, name: str, ctype: str, cpptype: str, **kwargs) -> None:
        self.name = name
        self.ctype = ctype
        self.cpptype = cpptype
        self.opaque_type = self.ctype + '_t'
        self.cpp_class = CPPClass(name, ctype)
        add_handle(name, ctype, cpptype, **kwargs)
        cpp_type_map[cpptype] = name

    def cname(self, name: str) -> str:
        return self.ctype + '_' + name

    def substitute(self, s: str, **kwargs) -> str:
        return Template(s).safe_substitute(name=self.name,
                                           ctype=self.ctype,
                                           cpptype=self.cpptype,
                                           opaque_type=self.opaque_type,
                                           **kwargs)

    def constructor(self,
                    name: str,
                    params: Optional[List[Parameter]] = None,
                    fname: Optional[str] = None,
                    invoke: Optional[str] = None,
                    **kwargs) -> 'Handle':
        create = self.substitute('allocate<${cpptype}>($@)')
        if fname:
            create = self.substitute('allocate<${cpptype}>(${fname}($@))',
                                     fname=fname)

        f = add_function(self.cname(name),
                         params=params,
                         invoke=invoke or create,
                         returns=self.cpptype + '*',
                         return_name=self.name,
                         **kwargs)
        self.cpp_class.add_constructor(name, f)
        return self

    def method(self,
               name: str,
               params: Optional[List[Parameter]] = None,
               fname: Optional[str] = None,
               invoke: Optional[str] = None,
               cpp_name: Optional[str] = None,
               const: Optional[bool] = None,
               **kwargs) -> 'Handle':
        cpptype = self.cpptype
        if const:
            cpptype = Type(cpptype).add_const().str()
        p = Parameter(self.name, cpptype)
        args = to_template_vars(params or [])
        f = add_function(self.cname(name),
                         params=[p] + (params or []),
                         invoke=invoke
                         or self.substitute('${var}.${fname}(${args})',
                                            var=template_var(self.name),
                                            fname=fname or name,
                                            args=args),
                         **kwargs)
        self.cpp_class.add_method(cpp_name or name, f)
        return self

    def function(self, name, params=None, **kwargs):
        add_function(self.cname(name), params=params, **kwargs)
        return self

    def add_cpp_class(self) -> None:
        cpp_classes.append(self.cpp_class)


interface_handle_definition = Template('''
extern "C" struct ${ctype};
struct ${ctype} {
    template<class... Ts>
    ${ctype}(void* p, ${copier} c, ${deleter} d,  const char* obj_typename, Ts&&... xs)
    : object_ptr(p, obj_typename, c, d), xobject(std::forward<Ts>(xs)...)
    {}
    manage_generic_ptr<${copier}, ${deleter}> object_ptr = nullptr;
    ${cpptype} xobject;
    ${functions}
};
''')

c_api_virtual_impl = Template('''
${return_type} ${name}(${params}) const
{
    if (${fname} == nullptr)
        throw std::runtime_error("${name} function is missing.");
    ${output_decls}
    std::array<char, 256> exception_msg;
    exception_msg.front() = '\\0';
    auto api_error_result = ${fname}(${args});
    if (api_error_result != ${success}) {
        const std::string exception_str(exception_msg.data()); 
        throw std::runtime_error("Error in ${name} of: " + std::string(object_ptr.obj_typename) + ": " + exception_str);
    }
    return ${output};
}
''')


def generate_virtual_impl(f: Function, fname: str) -> str:
    success = success_type
    name = f.name
    return_type = 'void'
    output_decls = ''
    output = ''
    largs = []
    lparams = []
    if f.returns:
        return_type = f.returns.type.str()
        output_decls = '\n'.join(f.returns.virtual_output_declarations())
        largs += f.returns.virtual_output_args()
        output = f.returns.virtual_output()
    largs += [arg for p in f.params for arg in p.virtual_arg()]
    lparams += [
        p.virtual_param() for p in f.params if not (p.this or p.hidden)
    ]
    args = ', '.join(largs)
    params = ', '.join(lparams)
    return c_api_virtual_impl.substitute(locals())


class Interface(Handle):
    def __init__(self, name: str, ctype: str, cpptype: str) -> None:
        super().__init__(name, ctype, cpptype, skip_def=True)
        self.ifunctions: List[Function] = []
        self.members: List[str] = []

    def mname(self, name: str) -> str:
        return name + "_f"

    def constructor(  # type: ignore
            self,
            name: str,
            params: Optional[List[Parameter]] = None,
            **kwargs) -> 'Interface':
        create = self.substitute('allocate<${opaque_type}>($@)')

        initial_params = gparams(obj='void*',
                                 c=self.cname('copy'),
                                 d=self.cname('delete'))

        add_function(self.cname(name),
                     params=initial_params + (params or []),
                     invoke=create,
                     returns=self.opaque_type,
                     return_name=self.name,
                     **kwargs)
        return self

    def method(self, *args, **kwargs) -> 'Interface':
        super().method(*args, **kwargs)
        return self

    def virtual(self,
                name: str,
                params: Optional[List[Parameter]] = None,
                const: Optional[bool] = None,
                **kwargs) -> 'Interface':

        # Add this parameter to the function
        this = Parameter('obj', 'void*', this=True)
        this.virtual_read = ['object_ptr.data']
        exception_msg = Parameter('exception_msg', 'char*', hidden=True)
        exception_msg.virtual_read = ['${name}.data()']
        exception_msg_size = Parameter('exception_msg_size',
                                       'size_t',
                                       hidden=True)
        exception_msg_size.virtual_read = ['exception_msg.size()']
        f = Function(name,
                     params=[this, exception_msg, exception_msg_size] +
                     (params or []),
                     virtual=True,
                     **kwargs)
        self.ifunctions.append(f)

        add_function(self.cname('set_' + name),
                     params=gparams(obj=self.opaque_type,
                                    input=self.cname(name)),
                     invoke='${{obj}}->{name} = ${{input}}'.format(
                         name=self.mname(name)))
        return self

    def generate_function(self, f: Function):
        cname = self.cname(f.name)
        mname = self.mname(f.name)
        function = generate_virtual_impl(f, fname=mname)
        return f"{cname} {mname} = nullptr;{function}"

    def generate(self):
        required_functions = [
            Function('copy',
                     params=gparams(out='void**', input='void*'),
                     virtual=True),
            Function('delete', params=gparams(input='void*'), virtual=True)
        ]
        for f in self.ifunctions + required_functions:
            f.update()
        c_header_preamble.extend([
            f.get_cfunction().generate_function_pointer(self.cname(f.name))
            for f in self.ifunctions + required_functions
        ])
        function_list = [self.generate_function(f) for f in self.ifunctions]
        ctype = self.ctype
        cpptype = self.cpptype
        copier = self.cname('copy')
        deleter = self.cname('delete')
        functions = '\n'.join(function_list)

        c_api_body_preamble.append(
            interface_handle_definition.substitute(locals()))


def handle(ctype: str,
           cpptype: str,
           name: Optional[str] = None,
           ref: Optional[bool] = None) -> Callable:
    def with_handle(f):
        n = name or f.__name__
        h = Handle(n, ctype, cpptype, ref=ref)
        f(h)
        h.add_cpp_class()

        @wraps(f)
        def decorated(*args, **kwargs):
            return f(*args, **kwargs)

        return decorated

    return with_handle


def interface(ctype: str, cpptype: str,
              name: Optional[str] = None) -> Callable:
    def with_interface(f):
        n = name or f.__name__
        h = Interface(n, ctype, cpptype)
        f(h)
        h.generate()

        @wraps(f)
        def decorated(*args, **kwargs):
            return f(*args, **kwargs)

        return decorated

    return with_interface


def template_eval(template, **kwargs):
    start = '<%'
    end = '%>'
    escaped = (re.escape(start), re.escape(end))
    mark = re.compile('%s(.*?)%s' % escaped, re.DOTALL)
    for key in kwargs:
        exec('%s = %s' % (key, kwargs[key]))
    for item in mark.findall(template):
        e = eval(item.strip())
        template = template.replace(start + item + end, str(e))
    return template


def run(path: Union[Path, str]) -> str:
    return template_eval(open(path).read())


if __name__ == "__main__":
    sys.modules['api'] = sys.modules['__main__']
    runpy.run_path(sys.argv[1])
    if len(sys.argv) > 2:
        r = run(sys.argv[2])
        sys.stdout.write(r)
    else:
        sys.stdout.write(generate_c_header())
        sys.stdout.write(generate_c_api_body())
        # sys.stdout.write(generate_cpp_header())



#####################################################################################
# The MIT License (MIT)
#
# Copyright (c) 2015-2023 Advanced Micro Devices, Inc. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#####################################################################################
import api, argparse, os, runpy, subprocess, sys, te
from pathlib import Path

clang_format_path = Path('clang-format.exe' if os.name ==
                         'nt' else '/opt/rocm/llvm/bin/clang-format')
work_dir = Path().cwd()
src_dir = (work_dir / '../src').absolute()
migraphx_py_path = src_dir / 'api/migraphx.py'


def clang_format(buffer, **kwargs):
    return subprocess.run(f'{clang_format_path} -style=file',
                          capture_output=True,
                          shell=True,
                          check=True,
                          input=buffer.encode('utf-8'),
                          cwd=work_dir,
                          **kwargs).stdout.decode('utf-8')


def api_generate(input_path: Path, output_path: Path):
    with open(output_path, 'w') as f:
        f.write(clang_format(api.run(input_path)))


def te_generate(input_path: Path, output_path: Path):
    with open(output_path, 'w') as f:
        f.write(clang_format(te.run(input_path)))


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-f', '--clang-format', type=Path)
    args = parser.parse_args()

    global clang_format_path
    if args.clang_format:
        clang_format_path = args.clang_format

    if not clang_format_path.is_file():
        print(f"{clang_format_path}: invalid path or not installed",
              file=sys.stderr)
        return

    try:
        files = Path('include').absolute().iterdir()
        for f in [f for f in files if f.is_file()]:
            te_generate(f, src_dir / f'include/migraphx/{f.name}')
        runpy.run_path(str(migraphx_py_path))
        api_generate(work_dir / 'api/migraphx.h',
                     src_dir / 'api/include/migraphx/migraphx.h')
        print('Finished generating header migraphx.h')
        api_generate(work_dir / 'api/api.cpp', src_dir / 'api/api.cpp')
        print('Finished generating source api.cpp')
    except subprocess.CalledProcessError as ex:
        if ex.stdout:
            print(ex.stdout.decode('utf-8'))
        if ex.stderr:
            print(ex.stdout.decode('utf-8'))
        print(f"Command '{ex.cmd}' returned {ex.returncode}")
        raise


if __name__ == "__main__":
    main()



#####################################################################################
# The MIT License (MIT)
#
# Copyright (c) 2015-2024 Advanced Micro Devices, Inc. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#####################################################################################
import migraphx


def test_autocast_fp8_1():
    p1 = migraphx.program()
    m1 = p1.get_main_module()
    x = m1.add_parameter("x", shape=migraphx.shape(type='fp8e4m3fnuz_type'))
    y = m1.add_parameter("y", shape=migraphx.shape(type='fp8e4m3fnuz_type'))
    sum_op = m1.add_instruction(migraphx.op("add"), [x, y])
    m1.add_return([sum_op])

    m1 = migraphx.autocast_fp8(p1)

    p2 = migraphx.program()
    m2 = p2.get_main_module()
    y_fp32 = m2.add_parameter("y", shape=migraphx.shape(type='float_type'))
    x_fp32 = m2.add_parameter("x", shape=migraphx.shape(type='float_type'))

    y_fp8 = m2.add_instruction(
        migraphx.op("convert",
                    target_type=int(migraphx.shape.type_t.fp8e4m3fnuz_type)),
        [y_fp32])
    x_fp8 = m2.add_instruction(
        migraphx.op("convert",
                    target_type=int(migraphx.shape.type_t.fp8e4m3fnuz_type)),
        [x_fp32])

    sum_fp8 = m2.add_instruction(migraphx.op("add"), [x_fp8, y_fp8])
    sum_fp32 = m2.add_instruction(
        migraphx.op("convert",
                    target_type=int(migraphx.shape.type_t.float_type)),
        [sum_fp8])

    m2.add_return([sum_fp32])
    assert p1 == p2


def test_autocast_fp8_2():
    p1 = migraphx.program()
    m1 = p1.get_main_module()
    x = m1.add_parameter("x", shape=migraphx.shape(type='float_type'))
    y = m1.add_parameter("y", shape=migraphx.shape(type='float_type'))
    sum = m1.add_instruction(migraphx.op("add"), [x, y])
    m1.add_return([sum])

    m1 = migraphx.autocast_fp8(p1)

    p2 = p1
    assert p1 == p2


if __name__ == "__main__":
    test_autocast_fp8_1()
    test_autocast_fp8_2()



#####################################################################################
# The MIT License (MIT)
#
# Copyright (c) 2015-2024 Advanced Micro Devices, Inc. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
#####################################################################################
import abc
import os
from optimum.exporters.onnx import main_export
from ..utils import download


class BaseModel(abc.ABC):
    @property
    @abc.abstractmethod
    def model_id(self):
        pass

    @staticmethod
    @abc.abstractmethod
    def name():
        pass

    @property
    def task(self):
        return "auto"

    @abc.abstractmethod
    def get_model(self, folder, force_download):
        pass

    @abc.abstractmethod
    def preprocess(self, *args, **kwargs):
        pass

    def is_decoder(self):
        return False

    def is_diffuser(self):
        return False


class DecoderModel(BaseModel):
    def is_decoder(self):
        return True

    @abc.abstractmethod
    def decode_step(self, *args, **kwargs):
        pass


class DiffusionModel(BaseModel):
    def is_diffuser(self):
        return True

    @abc.abstractmethod
    def get_models(self, folder, models, force_download):
        pass

    @abc.abstractmethod
    def text_preprocess(self, *args, **kwargs):
        pass

    @abc.abstractmethod
    def text_preprocess_2(self, *args, **kwargs):
        pass

    @abc.abstractmethod
    def image_preprocess(self, *args, **kwargs):
        pass

    @abc.abstractmethod
    def scheduler(self, *args, **kwargs):
        pass


class SingleModelDownloadMixin(object):
    def get_model(self, output_folder, force_download=False):
        filepath = f"{output_folder}/model.onnx"
        if force_download or not os.path.isfile(filepath):
            print(f"Download model from {self.model_id} to {filepath}")
            download(self.model_id, filepath)
        return filepath


class OptimumHFModelDownloadMixin(object):
    def get_model(self, folder, force_download=False):
        filepath = f"{folder}/model.onnx"
        if force_download or not os.path.isfile(filepath):
            print(f"Download model from {self.model_id} to {filepath}")
            main_export(
                self.model_id,
                output=folder,
                monolith=True,  # forces export into one model
                task=self.task)
        return filepath



#####################################################################################
# The MIT License (MIT)
#
# Copyright (c) 2015-2022 Advanced Micro Devices, Inc. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#####################################################################################
# -------------------------------------------------------------------------
# Copyright (c) Advanced Micro Devices. All rights reserved.
# Licensed under the MIT License.
# --------------------------------------------------------------------------
"""
Implements ONNX's backend API.
"""
import sys
if sys.version_info < (3, 0):
    sys.exit()

from onnx import ModelProto
from onnx.checker import check_model
from onnx.backend.base import Backend
import migraphx
from onnx_migraphx.backend_rep import MIGraphXBackendRep


def get_device():
    return ("CPU", "GPU")


class MIGraphXBackend(Backend):
    _device = "GPU"
    _input_names = []
    _prog_string = ""

    @classmethod
    def set_device(cls, device):
        cls._device = device
    """
    Implements
    `ONNX's backend API <https://github.com/onnx/onnx/blob/master/docs/ImplementingAnOnnxBackend.md>`_
    with *ONNX Runtime*.
    The backend is mostly used when you need to switch between
    multiple runtimes with the same API.
    `Importing models from ONNX to Caffe2 <https://github.com/onnx/tutorials/blob/master/tutorials/OnnxCaffe2Import.ipynb>`_
    shows how to use *caffe2* as a backend for a converted model.
    Note: This is not the official Python API.
    """  # noqa: E501

    @classmethod
    def get_program(cls):
        return cls._prog_string

    @classmethod
    def is_compatible(cls, model, device=None, **kwargs):
        """
        Return whether the model is compatible with the backend.

        :param model: unused
        :param device: None to use the default device or a string (ex: `'CPU'`)
        :return: boolean
        """
        device = cls._device
        return cls.supports_device(device)

    @classmethod
    def supports_device(cls, device):
        """
        Check whether the backend is compiled with particular device support.
        In particular it's used in the testing suite.
        """
        return device in get_device()

    @classmethod
    def prepare(cls, model, device=None, **kwargs):
        """
        Load the model and creates a :class:`migraphx.program`
        ready to be used as a backend.

        :param model: ModelProto (returned by `onnx.load`),
            string for a filename or bytes for a serialized model
        :param device: requested device for the computation,
            None means the default one which depends on
            the compilation settings
        :param kwargs: see :class:`onnxruntime.SessionOptions`
        :return: :class:`migraphx.program`
        """
        if isinstance(model, MIGraphXBackendRep):
            return model
        elif isinstance(model, migraphx.program):
            return MIGraphXBackendRep(model, cls._input_names)
        elif isinstance(model, (str, bytes)):
            if device is not None and not cls.supports_device(device):
                raise RuntimeError(
                    "Incompatible device expected '{0}', got '{1}'".format(
                        device, get_device()))
            inf = migraphx.parse_onnx_buffer(model)
            cls._prog_string = str("\nPython =\n{}\nProgram =\n{}".format(
                inf.to_py(), inf))
            device = cls._device
            cls._input_names = inf.get_parameter_names()
            inf.compile(migraphx.get_target(device.lower()))
            cls._prog_string = cls._prog_string + str(
                "\nCompiled program =\n{}".format(inf))

            return cls.prepare(inf, device, **kwargs)
        else:
            # type: ModelProto
            check_model(model)
            bin = model.SerializeToString()
            return cls.prepare(bin, device, **kwargs)

    @classmethod
    def run_model(cls, model, inputs, device=None, **kwargs):
        """
        Compute the prediction.

        :param model: :class:`migraphx.program` returned
            by function *prepare*
        :param inputs: inputs
        :param device: requested device for the computation,
            None means the default one which depends on
            the compilation settings
        :param kwargs: see :class:`migraphx.program`
        :return: predictions
        """
        rep = cls.prepare(model, device, **kwargs)
        return rep.run(inputs, **kwargs)

    @classmethod
    def run_node(cls, node, inputs, device=None, outputs_info=None, **kwargs):
        '''
        This method is not implemented as it is much more efficient
        to run a whole model than every node independently.
        '''
        raise NotImplementedError(
            "It is much more efficient to run a whole model than every node independently."
        )


is_compatible = MIGraphXBackend.is_compatible
prepare = MIGraphXBackend.prepare
run = MIGraphXBackend.run_model
supports_device = MIGraphXBackend.supports_device



#####################################################################################
# The MIT License (MIT)
#
# Copyright (c) 2015-2024 Advanced Micro Devices, Inc. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
#####################################################################################
from .base import BaseDataset
from collections import namedtuple

Prompt = namedtuple('Prompt', ['name', 'prompt', 'negative_prompt'])

# source: https://huggingface.co/spaces/google/sdxl/blob/main/app.py#L19-L70
style_list = [
    Prompt(
        name="Cinematic",
        prompt=
        "cinematic still, emotional, harmonious, vignette, highly detailed, high budget, bokeh, cinemascope, moody, epic, gorgeous, film grain, grainy",
        negative_prompt=
        "anime, cartoon, graphic, text, painting, crayon, graphite, abstract, glitch, deformed, mutated, ugly, disfigured",
    ),
    Prompt(
        name="Photographic",
        prompt=
        "cinematic photo, 35mm photograph, film, bokeh, professional, 4k, highly detailed",
        negative_prompt=
        "drawing, painting, crayon, sketch, graphite, impressionist, noisy, blurry, soft, deformed, ugly",
    ),
    Prompt(
        name="Anime",
        prompt=
        "anime artwork, anime style, key visual, vibrant, studio anime,  highly detailed",
        negative_prompt=
        "photo, deformed, black and white, realism, disfigured, low contrast",
    ),
    Prompt(
        name="Manga",
        prompt=
        "manga style, vibrant, high-energy, detailed, iconic, Japanese comic style",
        negative_prompt=
        "ugly, deformed, noisy, blurry, low contrast, realism, photorealistic, Western comic style",
    ),
    Prompt(
        name="Digital Art",
        prompt=
        "concept art, digital artwork, illustrative, painterly, matte painting, highly detailed",
        negative_prompt="photo, photorealistic, realism, ugly",
    ),
    Prompt(
        name="Pixel art",
        prompt="pixel-art, low-res, blocky, pixel art style, 8-bit graphics",
        negative_prompt=
        "sloppy, messy, blurry, noisy, highly detailed, ultra textured, photo, realistic",
    ),
    Prompt(
        name="Fantasy art",
        prompt=
        "ethereal fantasy concept art, magnificent, celestial, ethereal, painterly, epic, majestic, magical, fantasy art, cover art, dreamy",
        negative_prompt=
        "photographic, realistic, realism, 35mm film, dslr, cropped, frame, text, deformed, glitch, noise, noisy, off-center, deformed, cross-eyed, closed eyes, bad anatomy, ugly, disfigured, sloppy, duplicate, mutated, black and white",
    ),
    Prompt(
        name="Neonpunk",
        prompt=
        "neonpunk style, cyberpunk, vaporwave, neon, vibes, vibrant, stunningly beautiful, crisp, detailed, sleek, ultramodern, magenta highlights, dark purple shadows, high contrast, cinematic, ultra detailed, intricate, professional",
        negative_prompt=
        "painting, drawing, illustration, glitch, deformed, mutated, cross-eyed, ugly, disfigured",
    ),
    Prompt(
        name="3D Model",
        prompt=
        "professional 3d model, octane render, highly detailed, volumetric, dramatic lighting",
        negative_prompt="ugly, deformed, noisy, low poly, blurry, painting",
    ),
]


class StylePrompts(BaseDataset):
    @property
    def url(self):
        return ""

    @property
    def split(self):
        return ""

    @staticmethod
    def name():
        return "style-prompts"

    def __iter__(self):
        print(f"Load dataset for {self.name()}")
        self.dataset = iter(style_list)
        return self.dataset

    def __next__(self):
        return next(self.dataset)

    def transform(self, inputs, data, prepocess_fn):
        result = prepocess_fn(data.prompt, data.negative_prompt)
        inputs, keys = sorted(inputs), sorted(list(result.keys()))
        assert inputs == keys, f"{inputs = } == {keys = }"
        # The result should be a simple dict, the preproc returns a wrapped class, dict() will remove it
        return dict(result)



#  The MIT License (MIT)
#
#  Copyright (c) 2015-2024 Advanced Micro Devices, Inc. All rights reserved.
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the 'Software'), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in
#  all copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
#  THE SOFTWARE.

from argparse import ArgumentParser
import onnx
import os


def argparser():
    parser = ArgumentParser()
    parser.add_argument("-i", "--model_path", required=True)
    parser.add_argument("-o", "--output_path", required=True)
    return parser.parse_args()


def add_nodes_as_output(model, nodes):
    shape_info = onnx.shape_inference.infer_shapes(model)
    output_nodes = [
        node for node in shape_info.graph.value_info if node.name in nodes
    ]
    model.graph.output.extend(output_nodes)
    onnx.checker.check_model(model)
    return model


def modify_model(model_path, output_path):
    model = onnx.load(model_path)
    model = add_nodes_as_output(model, ["hidden_states"])
    os.makedirs(os.path.dirname(output_path), exist_ok=True)
    onnx.save(model, output_path)


if __name__ == "__main__":
    args = argparser()
    modify_model(**vars(args))



#!/usr/bin/env python3
#####################################################################################
#  The MIT License (MIT)
#
#  Copyright (c) 2015-2025 Advanced Micro Devices, Inc. All rights reserved.
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in
#  all copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
#  THE SOFTWARE.
#####################################################################################
#################################### GUIDE ##########################################
#####################################################################################
# This Check_Stamped script is triggered by the Github workflows when a pull request is created.
# It works by generating a list of files that have been modified/created between the current
# up-to-date Develop Branch from MIGraphx and the Pull Request Branch via GIT DIFF.
# The script then checks that each file has the current year in the license stamp,
# with the assumption being that any modifications/creations will need to be stamped to the
# year that the modification/creation was made.
#####################################################################################

import sys
import argparse
import os
from stamp_status import StampStatus, stamp_check, current_year
from git_tools import get_changed_files

SUPPORTED_FILE_TYPES = (".cpp", ".hpp", ".h", ".ipynb", ".py", ".txt", ".sh",
                        ".bsh", "LICENSE", ".cmake")
IGNORED_FILES = ("digits.txt", "Dockerfile", "Jenkinsfile",
                 "imagenet_classes.txt", '')


def check_file(file, debug=False):
    try:
        with open(file, 'r') as f:
            content = f.read()
    except (OSError, UnicodeDecodeError) as e:
        if debug:
            print(f"{file}: Skipping ({e})")
        return StampStatus.ERROR_READING

    year = current_year()
    return stamp_check(file, year, content, debug=debug)


def print_status(status):
    files = status.files
    if files:
        print(f"\n{'Error' if status.error else 'Warning'}: "
              f"\n{len(files)} {status.label} files:\n{files}")
        return status.error
    return False


def main(args):
    files = get_changed_files(args.against)
    if args.debug:
        print(f"Changed files vs {args.against}: {files}")

    for file in files:
        filename = os.path.basename(file)

        # Assign appropriate StampStatus based on filename or content
        if filename in IGNORED_FILES:
            status = StampStatus.IGNORED
        elif not filename.endswith(SUPPORTED_FILE_TYPES):
            status = StampStatus.UNSUPPORTED
        else:
            status = check_file(file, args.debug)

        status.files.append(file)

    has_errors = any([
        print_status(StampStatus.UNSUPPORTED),
        print_status(StampStatus.IGNORED),
        print_status(StampStatus.NOT_STAMPED),
        print_status(StampStatus.WRONG_YEAR)
    ])

    if has_errors:
        sys.exit(1)
    else:
        print("Success: All files properly stamped.")


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument('against', default='origin/develop', nargs='?')
    parser.add_argument("-d",
                        "--debug",
                        action="store_true",
                        help="Enable debug output")
    args = parser.parse_args()
    main(args)



#####################################################################################
# The MIT License (MIT)
#
# Copyright (c) 2015-2023 Advanced Micro Devices, Inc. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#####################################################################################
import migraphx


def test_add_fill():
    p = migraphx.program()
    mm = p.get_main_module()
    x = mm.add_literal(
        migraphx.fill_argument(migraphx.shape(type='float_type', lens=[3, 3]),
                               1))
    y = mm.add_literal(
        migraphx.fill_argument(migraphx.shape(type='float_type', lens=[3, 3]),
                               2))
    add_op = mm.add_instruction(migraphx.op("add"), [x, y])
    mm.add_return([add_op])
    p.compile(migraphx.get_target("ref"))
    params = {}
    output = p.run(params)[-1].tolist()
    assert output == list([3.0] * 9)


def test_add_create():
    p = migraphx.program()
    mm = p.get_main_module()
    x = mm.add_literal(
        migraphx.create_argument(
            migraphx.shape(type='float_type', lens=[2, 2]), [1, 2, 3, 4]))
    y = mm.add_literal(
        migraphx.create_argument(
            migraphx.shape(type='float_type', lens=[2, 2]), [5, 6, 7, 8]))
    add_op = mm.add_instruction(migraphx.op("add"), [x, y])
    mm.add_return([add_op])
    p.compile(migraphx.get_target("ref"))
    params = {}
    output = p.run(params)[-1].tolist()
    assert output == list([6, 8, 10, 12])


if __name__ == "__main__":
    test_add_fill()
    test_add_create()



#####################################################################################
# The MIT License (MIT)
#
# Copyright (c) 2015-2024 Advanced Micro Devices, Inc. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#####################################################################################
# Inference with ONNX Runtime
import onnxruntime
import time
from torchvision import models
import torch
from PIL import Image
import numpy as np
import argparse
import os
import subprocess

#Use most upto date weights
resnet50 = models.resnet50(weights=models.ResNet50_Weights.DEFAULT,
                           progress=True).eval()

# Download ImageNet labels
#!curl -o imagenet_classes.txt https://raw.githubusercontent.com/pytorch/hub/master/imagenet_classes.txt


def parse_input_args():
    parser = argparse.ArgumentParser()

    parser.add_argument(
        "--fp16",
        action="store_true",
        required=False,
        default=False,
        help='Perform fp16 quantization on the model before running inference',
    )

    parser.add_argument(
        "--image_dir",
        required=False,
        default="../dataset/images",
        help='Target DIR for images to infer. Default is ../dataset/images')

    parser.add_argument("--batch",
                        required=False,
                        default=1,
                        help='Batch size of images per inference',
                        type=int)

    parser.add_argument("--run",
                        required=False,
                        default=100,
                        help='Number of runs for each batched inference.',
                        type=int)

    parser.add_argument("--top",
                        required=False,
                        default=1,
                        help='Show top K of inference results',
                        type=int)

    parser.add_argument(
        "--QPS",
        action="store_true",
        required=False,
        default=False,
        help=
        'Show inference result in Queries-Per-Second QPS instead of inference duration (milliseconds)',
    )

    parser.add_argument(
        "--verbose",
        action="store_true",
        required=False,
        default=False,
        help='Show verbose output',
    )

    return parser.parse_args()


def get_image_list_in_dir(dir):
    proc = subprocess.run("ls", shell=True, stdout=subprocess.PIPE, cwd=dir)
    fileList = proc.stdout.decode().split('\n')
    return fileList


def softmax(x):
    """Compute softmax values for each sets of scores in x."""
    e_x = np.exp(x - np.max(x))
    return e_x / e_x.sum()


def run_sample(session,
               categories,
               latency,
               inputs,
               topk,
               batch_size,
               verbose=False):
    io_binding = session.io_binding()
    io_binding.bind_cpu_input('input', inputs.cpu().detach().numpy())
    io_binding.bind_output('output', 'cuda')
    start = time.time()
    session.run_with_iobinding(io_binding)
    latency.append(time.time() - start)
    ort_outputs = io_binding.copy_outputs_to_cpu()[0]

    #Get prediction for each item
    for i in range(batch_size):
        output = ort_outputs[i].copy().flatten()
        output = softmax(output)  # this is optional
        top5_catid = np.argsort(-output)[:topk]

        if verbose:
            for catid in top5_catid:
                print(categories[catid], output[catid])


#Filter Images based on image preprocessing
def preprocess_images(image_dir, image_width, image_height, batch_size,
                      verbose):
    if verbose:
        print("Read from dir " + image_dir)

    fileList = get_image_list_in_dir(image_dir)

    if verbose:
        print(fileList)

    #Setup input data feed
    input_batch = torch.empty(batch_size, 3, image_width, image_height)

    bad_image = 0  # Bad images skipped in data set from image list
    bad_img_list = []  # Keep track of bad images we can't preprocess
    img_count = 0  # Number of preprocced images
    i = 0
    while i in range(len(fileList)):
        img = fileList[img_count + bad_image]
        if img == '':
            break

        if verbose:
            print("Preprocess: " + img)

        input_image = Image.open(str(image_dir + "/" + img))
        preprocess = models.ResNet50_Weights.IMAGENET1K_V1.transforms()
        try:
            input_tensor = preprocess(input_image)
        except RuntimeError:
            if verbose:
                print("Skipping " + img)
            bad_image = bad_image + 1
            bad_img_list.append(img)
            continue

        input_batch[img_count] = input_tensor.unsqueeze(
            0)  # create a mini-batch as expected by the model

        img_count = img_count + 1
        if img_count >= batch_size:
            break

    if img_count < batch_size:
        print("Warning Batch size " + str(batch_size) + "Requested but only " +
              str(img_count) + "Images from dataset")
        print(bad_image + "Images caused RunTimeErrors during preprocessing")
        if verbose:
            print("List of bad Images:\n" + str(bad_img_list))

    return input_batch


def main():
    flags = parse_input_args()

    if flags.verbose:
        print(flags)

    if flags.verbose:
        print("Reading in Imagenet classes")

    # Read the categories
    with open("../dataset/imagenet_classes.txt", "r") as f:
        categories = [s.strip() for s in f.readlines()]

    if flags.verbose:
        print("Getting Exported Model from Torch")

    # Export the model to ONNX
    image_height = 224
    image_width = 224
    image_chan = 3
    x = torch.randn(flags.batch,
                    image_chan,
                    image_height,
                    image_width,
                    requires_grad=True)
    resnet50(x)
    torch.onnx.export(
        resnet50,  # model being run
        x,  # model input (or a tuple for multiple inputs)
        "resnet50.onnx",  # where to save the model (can be a file or file-like object)
        export_params=
        True,  # store the trained parameter weights inside the model file
        opset_version=12,  # the ONNX version to export the model to
        do_constant_folding=
        True,  # whether to execute constant folding for optimization
        input_names=['input'],  # the model's input names
        output_names=['output'])  # the model's output names

    # Quantize the model
    if flags.fp16:
        if flags.verbose:
            print("FP16 Quantization Enabled")
        os.environ["ORT_MIGRAPHX_FP16_ENABLE"] = "1"  # Enable FP16 precision
    else:
        os.environ["ORT_MIGRAPHX_FP16_ENABLE"] = "0"  # Disable FP32 precision

    session_ops = onnxruntime.SessionOptions()
    if flags.verbose:
        session_ops.log_verbosity_level = 0
        session_ops.log_severity_level = 0

    session_fp32 = onnxruntime.InferenceSession(
        "resnet50.onnx",
        providers=['MIGraphXExecutionProvider'],
        sess_options=session_ops)

    if flags.verbose:
        print("Preprocessing Batched Images")

    # Preproccess and batch images
    input_batch = preprocess_images(flags.image_dir, image_height, image_width,
                                    flags.batch, flags.verbose)

    latency = []
    if flags.verbose:
        print("Running samples")
    for i in range(flags.run + 1):  # +1 for warm-up run
        run_sample(session_fp32, categories, latency, input_batch, flags.top,
                   flags.batch)

    if flags.verbose:
        print("Running Complete")
        print(latency)

    if flags.QPS:
        print("resnet50, Rate = {} QPS ".format(
            format((((flags.batch) / (sum(latency[1:]) / len(latency[1:])))),
                   '.2f')))

    else:
        print("resnet50, Averaage Execution time = {} ms".format(
            format(sum(latency[1:]) * 1000 / len(latency[1:]), '.2f')))


if __name__ == "__main__":
    main()



#####################################################################################
# The MIT License (MIT)
#
# Copyright (c) 2015-2024 Advanced Micro Devices, Inc. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
#####################################################################################
from .base import DiffusionModel
from transformers import AutoTokenizer
from diffusers import EulerDiscreteScheduler
from diffusers.image_processor import VaeImageProcessor
from optimum.exporters.onnx import main_export
import numpy as np
import os
from PIL import ImageOps


class OptimumHFDiffusionModelDownloadMixin(object):
    def get_models(self, folder, models, force_download=False):
        filepaths = [f"{folder}/{model}/model.onnx" for model in models]
        if force_download or not all(
                os.path.isfile(filepath) for filepath in filepaths):
            print(f"Download model from {self.model_id} to {filepaths}")
            main_export(
                self.model_id,
                output=folder,
                monolith=True,  # forces export into one model
                task=self.task)
        return filepaths


class AutoTokenizerHFMixin(object):

    _tokenizer = None
    _tokenizer_2 = None

    @property
    def tokenizer(self):
        if self._tokenizer is None:
            self._tokenizer = AutoTokenizer.from_pretrained(
                self.model_id, subfolder="tokenizer")
        return self._tokenizer

    @property
    def tokenizer_2(self):
        if self._tokenizer_2 is None:
            self._tokenizer_2 = AutoTokenizer.from_pretrained(
                self.model_id, subfolder="tokenizer_2")
        return self._tokenizer_2

    def text_preprocess(self, prompt, max_length=77, version_2=False):
        tokenizer = self.tokenizer_2 if version_2 else self.tokenizer
        return tokenizer(prompt,
                         padding='max_length',
                         max_length=max_length,
                         return_tensors="np")


class VAEImageProcessorHFMixin(object):

    _processor = None

    @property
    def processor(self):
        if self._processor is None:
            self._processor = VaeImageProcessor()
        return self._processor

    def image_preprocess(self, image_data):
        return self.processor.preprocess(image_data).numpy()


class EulerDiscreteSchedulerHFMixin(object):

    _scheduler = None

    @property
    def scheduler(self):
        if self._scheduler is None:
            self._scheduler = EulerDiscreteScheduler.from_pretrained(
                self.model_id, subfolder="scheduler")
        return self._scheduler


class StableDiffusion21(OptimumHFDiffusionModelDownloadMixin,
                        AutoTokenizerHFMixin, VAEImageProcessorHFMixin,
                        EulerDiscreteSchedulerHFMixin, DiffusionModel):
    def __init__(self):
        # it should be in pipeline exec order
        self.models = ('text_encoder', 'vae_encoder', 'unet', 'vae_decoder')

    @property
    def model_id(self):
        return "stabilityai/stable-diffusion-2-1"

    @staticmethod
    def name():
        return "stable-diffusion-2-1"

    def get_model(self, folder, force_download=False):
        return super().get_models(folder, self.models, force_download)

    def preprocess(self, *args, **kwargs):
        raise RuntimeError("Call text_preprocess or image_preprocess directly")

    def text_preprocess(self, *args, **kwargs):
        prompt, negative_prompt = args
        prompt_result = super().text_preprocess(prompt, **kwargs)
        negative_prompt_result = super().text_preprocess(
            negative_prompt, **kwargs)
        result = {
            'input_ids':
            np.concatenate(
                (prompt_result['input_ids'],
                 negative_prompt_result['input_ids'])).astype(np.int32)
        }
        return result

    def text_preprocess_2(self, *args, **kwargs):
        raise RuntimeError("No tokenizer_2 for SD21 model")

    def image_preprocess(self, *args, **kwargs):
        resized_image = ImageOps.fit(args[0], (512, 512))
        result = super().image_preprocess(resized_image, **kwargs)
        return {"sample": result}


class StableDiffusionXL(OptimumHFDiffusionModelDownloadMixin,
                        AutoTokenizerHFMixin, VAEImageProcessorHFMixin,
                        EulerDiscreteSchedulerHFMixin, DiffusionModel):
    def __init__(self):
        # it should be in pipeline exec order
        self.models = ('text_encoder', 'text_encoder_2', 'vae_encoder', 'unet',
                       'vae_decoder')

    @property
    def model_id(self):
        return "stabilityai/stable-diffusion-xl-base-1.0"

    @staticmethod
    def name():
        return "stable-diffusion-xl"

    def get_model(self, folder, force_download=False):
        return super().get_models(folder, self.models, force_download)

    def preprocess(self, *args, **kwargs):
        raise RuntimeError("Call text_preprocess or image_preprocess directly")

    def text_preprocess(self, *args, **kwargs):
        prompt, negative_prompt = args
        dtype = np.int64 if kwargs.get('version_2', False) else np.int32
        prompt_result = super().text_preprocess(prompt, **kwargs)
        negative_prompt_result = super().text_preprocess(
            negative_prompt, **kwargs)
        result = {
            'input_ids':
            np.concatenate((prompt_result['input_ids'],
                            negative_prompt_result['input_ids'])).astype(dtype)
        }
        return result

    def text_preprocess_2(self, *args, **kwargs):
        new_kwargs = dict(kwargs)
        new_kwargs['version_2'] = True
        return self.text_preprocess(*args, **new_kwargs)

    def image_preprocess(self, *args, **kwargs):
        resized_image = ImageOps.fit(args[0], (1024, 1024))
        result = super().image_preprocess(resized_image, **kwargs)
        return {"sample": result}
